SystemOrganization addCategory: #'OmniBrowser-Actors'!
SystemOrganization addCategory: #'OmniBrowser-Kernel'!
SystemOrganization addCategory: #'OmniBrowser-Morphic'!
SystemOrganization addCategory: #'OmniBrowser-Nodes'!
SystemOrganization addCategory: #'OmniBrowser-Notifications'!
SystemOrganization addCategory: #'OmniBrowser-Panels'!
SystemOrganization addCategory: #'OmniBrowser-Tests'!
SystemOrganization addCategory: #'OmniBrowser-Utilities'!

TextMorphEditor subclass: #OBTextMorphEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBTextMorphEditor commentStamp: 'cwp 1/7/2005 23:27' prior: 0!
OBTextMorphEditor overrides the TextMorphEditors handling of command keys, passing them along to its model for processing, rather than hard-coding their implementations.!

!OBTextMorphEditor methodsFor: 'actions' stamp: 'cwp 10/31/2004 01:52'!
browseIt
	| symbol |
	self lineSelectAndEmptyCheck: [^ self].
	(symbol _ self selectedSymbol) isNil ifTrue: [^ view flash].

	self send: #browseIt: toModelWith: {symbol} orDo: [super browseIt]! !

!OBTextMorphEditor methodsFor: 'actions' stamp: 'cwp 10/31/2004 00:40'!
implementorsOfIt
	"Open a senders browser on the selected selector"

	| selector |
	self lineSelectAndEmptyCheck: [^ self].
	(selector _ self selectedSelector) == nil ifTrue: [^ view flash].
	self send: #implementorsOfIt: toModelWith: {selector} orDo: [super sendersOfIt]! !

!OBTextMorphEditor methodsFor: 'actions' stamp: 'cwp 10/31/2004 01:10'!
referencesToIt

	| selector |
	self lineSelectAndEmptyCheck: [^ self].
	(selector _ self selectedSelector) == nil ifTrue: [^ view flash].
	self send: #referencesToIt: toModelWith: {selector} orDo: [super referencesToIt]! !

!OBTextMorphEditor methodsFor: 'actions' stamp: 'cwp 10/31/2004 00:26'!
sendersOfIt
	"Open a senders browser on the selected selector"

	| selector |
	self lineSelectAndEmptyCheck: [^ self].
	(selector _ self selectedSelector) == nil ifTrue: [^ view flash].
	self send: #sendersOfIt: toModelWith: {selector} orDo: [super sendersOfIt]! !

!OBTextMorphEditor methodsFor: 'model access' stamp: 'cwp 10/31/2004 00:24'!
send: aSelector toModelWith: args orDo: aBlock
	self terminateAndInitializeAround:
		[(model respondsTo: aSelector)
			ifTrue: [(model perform: aSelector withArguments: args)
						ifFalse: [view flash]]
			ifFalse: aBlock]! !

RectangleMorph subclass: #OBPane
	instanceVariableNames: 'model list button'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBPane commentStamp: 'cwp 1/7/2005 23:24' prior: 0!
An OBPane is the visual representation of a column in a browser. It contains a morph to display nodes (typically a PluggableListMorph) and (optionally) a morph for communicating with the column's filter. It's main responsibility is to lay out its submorphs as the filter controls are added and removed.

iVars:

model	- the OBColumn that controls the node list displayed in this pane
list		- the morph which displays the node list, usually a PluggableListMorph
button	- the morph which controls the column's filter, usually an OBRadioButtonBar.!

!OBPane class methodsFor: 'instance creation' stamp: 'cwp 2/12/2004 18:11'!
forColumn: aColumn
	^ self forColumn: aColumn withFilter: nil! !

!OBPane class methodsFor: 'instance creation' stamp: 'cwp 2/12/2004 18:12'!
forColumn: aColumn withFilter: aFilter
	^ self new setColumn: aColumn filter: aFilter! !

!OBPane methodsFor: 'updating' stamp: 'cwp 11/2/2004 00:53'!
addButton: aButton
	self hasButton ifTrue: [self removeMorph: button].
	button _ aButton.
	button height: self defaultButtonHeight.
	self 
		addMorph: button;
		adjustList;
		adjustButton! !

!OBPane methodsFor: 'updating' stamp: 'cwp 11/2/2004 00:50'!
adjustButton
	| inner |
	self hasButton ifTrue: 
		[inner _ self innerBounds.
		button bounds: (inner withTop: inner bottom - self buttonHeight)]! !

!OBPane methodsFor: 'updating' stamp: 'cwp 2/12/2004 18:35'!
adjustList
	self hasButton
			ifFalse: [list bounds: self innerBounds]
			ifTrue: [list bounds: (self innerBounds withHeight: (self height - self buttonHeight))]! !

!OBPane methodsFor: 'geometry' stamp: 'cwp 2/12/2004 18:32'!
bounds: aRectangle
	super bounds: aRectangle.
	self adjustList.
	self adjustButton.! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 11/2/2004 00:57'!
buttonHeight
	^ self hasButton
		ifTrue: [button height]
		ifFalse: [self defaultButtonHeight]
! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 11/2/2004 01:10'!
defaultButtonHeight
	^ 23! !

!OBPane methodsFor: 'testing' stamp: 'cwp 2/12/2004 18:35'!
hasButton
	^ button notNil! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:26'!
initGeometry
	self	
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		clipSubmorphs: true;
		color: Color transparent;
		cellInset: 0;
		borderWidth: 0;
		layoutPolicy: ProportionalLayout new.
		
	self addMorph: list.
	list bounds: self innerBounds.
		! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 2/12/2004 18:14'!
listMorphForColumn: aColumn
	^ (PluggableListMorph
		on: aColumn
		list: #list
		selected: #selection
		changeSelected: #selection:
		menu: #menu:
		keystroke: #keystroke:from:)
			borderWidth: 0! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:44'!
model: anObject
	"Set my model and make me me a dependent of the given object."

	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].
	model _ anObject.
! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 11/27/2004 21:19'!
noteNewOwner: aMorph
	self containingWindow ifNotNilDo: [:window | self adoptPaneColor: window paneColor]! !

!OBPane methodsFor: 'updating' stamp: 'cwp 2/12/2004 20:29'!
removeButton
	self hasButton 
		ifTrue: [self removeMorph: button.
				button _ nil.
				self adjustList]! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 19:17'!
setColumn: aColumn filter: aFilter
	self model: aColumn.
	list _ aColumn listMorph.
	self initGeometry.
	aFilter ifNotNil: [self addButton: aFilter buttonMorph].! !

!OBPane methodsFor: 'updating' stamp: 'cwp 2/12/2004 20:23'!
update: aSelector
	aSelector = #filter ifFalse: [^ self].
	(model wantsButton)
			ifTrue: [self addButton: model filter buttonMorph]
			ifFalse: [self removeButton]! !

RectangleMorph subclass: #OBPaneScroller
	instanceVariableNames: 'model sizing panes transform scrollBar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBPaneScroller commentStamp: 'cwp 3/5/2004 12:13' prior: 0!
Instances of OBPaneScroller contain the panes which represent columns in a browser. Their primary responsibilities are laying out panes to fit the space available and scrolling them horizontally when there isn't sufficient space. 

iVars:

sizing		- The number of panes which should exactly fit the available space.
		   	   During layout, the  width of the panes is determined accordingly.
transform	- A TransformMorph used for scrolling
scrollBar	- An OBHorizontalScrollBar used for scrolling!

!OBPaneScroller class methodsFor: 'as yet unclassified' stamp: 'cwp 11/17/2004 22:01'!
withModel: aModel
	^ self new model: aModel! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:10'!
adjustPaneHeight
	"This gets called after the scrollbar has been shown or hidden, to move the bottom
	of the panes to the right place."
	
	transform bounds: self innerBounds.
	transform submorphsDo: [:m | m bounds: (m bounds withHeight: self paneHeight)]
		! !

!OBPaneScroller methodsFor: 'private' stamp: 'cwp 11/23/2004 01:31'!
basicUpdatePanes
	panes := model ifNotNil: [model panes] ifNil: [Array new].		
	self clearPanes.
	panes do: [:ea | self pushPane: ea].
! !

!OBPaneScroller methodsFor: 'private' stamp: 'cwp 11/23/2004 01:14'!
basicUpdateSizing
	model ifNil: [sizing := 1] ifNotNil: [sizing := model sizing]! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 11/21/2004 23:45'!
bounds: aRectangle
	super bounds: aRectangle.
	self layoutWidgets.
	self layoutPanes.
	self setScrollDeltas.
! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 12/6/2003 17:06'!
clearPanes
	transform removeAllMorphs! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/26/2004 23:14'!
computeMorphWidths
	| paneWidths widths |
	paneWidths _ self paneWidthsToFit: self totalPaneWidth.
	widths _ OrderedCollection new.
	paneWidths do: [:w | widths add: w] separatedBy: [widths add: self separatorWidth].
	^ widths asArray
! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:06'!
doLayout
	self layoutWidgets.
	self layoutPanes.
	self hideOrShowScrollBar.
	self setScrollDeltas.
	self scrollToRight.! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 11/21/2004 22:24'!
fullLayout
	self layoutWidgets.
	self layoutPanes.
	self hideOrShowScrollBar.
	self setScrollDeltas.
	self scrollToRight.! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:44'!
hideOrShowScrollBar
	self isScrollable ifTrue: [self showScrollBar] ifFalse: [self hideScrollBar]! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 11:10'!
hideScrollBar
	self removeMorph: scrollBar.
	self adjustPaneHeight.! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 11/23/2004 01:28'!
initialize
	super initialize.
	self 
		color: Color red;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill.
	self 
		initializeTransform;
		initializeScrollbar.
	self startStepping.! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 2/8/2004 11:01'!
initializeScrollbar
	scrollBar _ OBScrollBar new model: self slotName: 'scrollBar'.
	scrollBar 
		borderWidth: 0;
		borderColor: #inset;
		height: self scrollBarHeight.
	self resizeScrollBar.
! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 2/8/2004 10:52'!
initializeTransform
	transform _ TransformMorph new.
	transform
		color: Color transparent;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill;
		disableTableLayout;
		bounds: super innerBounds.
	self addMorphBack: transform.
! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 10:54'!
innerBounds
	| rect |
	rect _ super innerBounds.
	^ self scrollBarIsVisible
		ifTrue: [rect withHeight: rect height - self scrollBarHeight - 1]
		ifFalse: [rect]! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:42'!
isScrollable
	^ self leftoverScrollRange > 0! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 01:22'!
layoutPanes
	| widths rect |
	widths _ self computeMorphWidths.
	rect _ 0@0 extent: (0 @ self paneHeight).
	transform submorphs 
					with: widths 
					do: [:m :w | 
						rect _ rect withWidth: w.
						m bounds: rect.
						rect _ rect translateBy: (w@0)]
						
! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 22:13'!
layoutWidgets
	| inner outer |
	outer _ super innerBounds.
	inner _ self innerBounds.
	transform bounds: inner.
	scrollBar bounds: ((inner left @ inner bottom + 1)
						corner: outer bottomRight)! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 21:04'!
leftoverScrollRange
	^ (self totalScrollRange - self innerBounds width roundTo: self scrollDeltaWidth) max: 0
! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 11/17/2004 22:03'!
model
	^model! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 11/17/2004 22:11'!
model: anObject
	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].
	model := anObject! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 11/17/2004 23:09'!
paneCount
	^ self panes size! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 12/6/2003 17:08'!
paneHeight
	^ transform bounds height! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 11/17/2004 23:09'!
panes
	^ panes ifNil: [self updatePanes. panes]! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/26/2004 23:14'!
paneWidthsToFit: limit
	| padded |
	padded _ Array new: self paneCount.
	padded atAllPut: (limit / self sizing) floor.
	(1 to: limit - padded sum) do: [:i | padded at: i put: (padded at: i) + 1].
	^ padded
	
	! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 11/23/2004 01:22'!
popPanes: count
	count * 2 timesRepeat: [transform removeMorph: transform lastSubmorph].
	panes removeLast: count! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 11/17/2004 22:46'!
pushPane: aMorph
	aMorph 
		borderWidth: 0;
		hResizing: #rigid;
		vResizing: #rigid;
		layoutInset: 0.
	transform hasSubmorphs ifTrue: [transform addMorphBack: self separator].
	transform addMorphBack: aMorph.
! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/23/2004 01:23'!
reclaimPanes
	| reclaimed |
	reclaimed := model reclaimPanes.
	reclaimed isZero	ifFalse: 
		[self 
		 	popPanes: reclaimed;
			basicUpdateSizing;
			layoutPanes;
			hideOrShowScrollBar;
			setScrollDeltas]! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:03'!
resizeScrollBar
	| inner outer |
	outer _ super innerBounds.
	inner _ outer withHeight: outer height - self scrollBarHeight - 1.
	scrollBar bounds: ((inner left @ inner bottom + 1)
						corner: outer bottomRight)! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 8/24/2003 14:29'!
scrollBarHeight
	^ 12! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:56'!
scrollBarIsVisible
	^ submorphs includes: scrollBar! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/21/2004 13:51'!
scrollBarValue: value
	transform hasSubmorphs ifFalse: [^ self].
	transform offset: (self leftoverScrollRange * value) rounded @ 0.! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/23/2003 16:21'!
scrollDeltaWidth
	^ 1! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 12/8/2003 21:42'!
scrollToRight
	^ scrollBar setValue: 1.! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 11/27/2004 01:03'!
separator
	^ BorderedSubpaneDividerMorph vertical 
		borderWidth: self separatorWidth / 2;
		color: model defaultBackgroundColor duller;
		borderRaised.! !

!OBPaneScroller methodsFor: 'defaults' stamp: 'cwp 8/24/2003 15:13'!
separatorWidth
	^ 4! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 21:14'!
setScrollDeltas
	| range interval value |
	transform hasSubmorphs ifFalse: [scrollBar interval: 1.0. ^ self].
	range _ self leftoverScrollRange.
	range = 0 ifTrue: [^ scrollBar interval: 1.0; setValue: 0].
	interval _ ((self innerBounds width) / self totalScrollRange) asFloat.
	value _ (transform offset x / range min: 1.0) asFloat.
	scrollBar interval: interval.
	scrollBar setValue: value.! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 11:11'!
showScrollBar
	self scrollBarIsVisible ifTrue: [^ self].
	self resizeScrollBar.
	self addMorphFront: scrollBar.
	self adjustPaneHeight.
	! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 11/22/2004 23:19'!
sizing
	^ sizing ifNil: [self updateSizing]! !

!OBPaneScroller methodsFor: 'stepping' stamp: 'cwp 11/21/2004 15:38'!
step
	self reclaimPanes! !

!OBPaneScroller methodsFor: 'stepping' stamp: 'cwp 11/24/2004 22:29'!
stepTime
	^ 250! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/25/2004 20:23'!
totalPaneWidth
	^ self innerBounds width - ((self sizing - 1) * self separatorWidth)! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 19:27'!
totalScrollRange
	| submorphBounds |
	submorphBounds := transform localSubmorphBounds ifNil: [^ 0].
	^ submorphBounds width
! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/23/2004 01:34'!
updatePanes
	| count |
	model ifNil: [panes := Array new. ^ self].
	count := panes ifNotNil: [panes size] ifNil: [0].
	self basicUpdatePanes.
	self basicUpdateSizing.	
	self layoutPanes.
	panes size = count
		ifFalse: [self hideOrShowScrollBar.
				self setScrollDeltas].
	panes size > count ifTrue: [self scrollToRight].
	^ panes! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/23/2004 01:14'!
updateSizing
	| old |
	old := sizing.
	self basicUpdateSizing.
	sizing = old ifFalse: [self layoutPanes].
	^sizing! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/17/2004 22:40'!
update: aSymbol
	aSymbol = #sizing ifTrue: [^ self updateSizing].
	aSymbol = #panes ifTrue: [^ self updatePanes].! !

RectangleMorph subclass: #OBRadioButtonBar
	instanceVariableNames: 'model buttons selection getListSelector getSelectionSelector setSelectionSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBRadioButtonBar commentStamp: 'cwp 1/7/2005 23:27' prior: 0!
An OBRadioButtonBar is similar to a PluggableListMorph except that it displays a row of buttons rather than a vertical list. Clicking on a button selects it.

model				- the model for this button bar
buttons			- a collection of OBButtonModels, which are derived from the model's list
selection			- the index of the currently selected button
getListSelector	 	- the message for getting the list of labels for the buttons
getSelectionSelector	- the message for getting the index of the currently selected item
setSelectionSelector	- the message for informing the model that a button has been clicked!

!OBRadioButtonBar class methodsFor: 'as yet unclassified' stamp: 'cwp 2/22/2004 16:44'!
on: aModel list: listSelector selected: selectionSelector changeSelected: changedSelector 
	^ self new
			on: aModel
			list: listSelector
			selected: selectionSelector
			changeSelected: changedSelector! !

!OBRadioButtonBar methodsFor: 'constructing' stamp: 'cwp 11/27/2004 20:58'!
adoptPaneColor: aColor
	self submorphs do: [:ea | ea onColor: aColor darker offColor: aColor lighter]! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 18:53'!
getSelectionIndex
	^ model perform: getSelectionSelector! !

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 11/27/2004 18:02'!
initGeometry
	self
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		rubberBandCells: true;
		borderWidth: 0.
! !

!OBRadioButtonBar methodsFor: 'callbacks' stamp: 'cwp 11/27/2004 19:18'!
isEnabled: aButton
	^ true! !

!OBRadioButtonBar methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 18:54'!
isSelected: aButton
	^ (buttons at: selection ifAbsent: [^ false]) == aButton! !

!OBRadioButtonBar methodsFor: 'accessing' stamp: 'cwp 11/27/2004 02:09'!
list
	buttons ifNil: 
			[| labels |
			labels := model perform: getListSelector.
			buttons := Array new: labels size.
			labels withIndexDo: 
					[:label :index | 
					buttons at: index put: (OBButtonModel withLabel: label inBar: self)].
			selection := self getSelectionIndex.
			self].
	^buttons collect: [:b | b label]! !

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 2/22/2004 16:51'!
model: aModel
	model ifNotNil: [model removeDependent: self].
	model _ aModel.
	model addDependent: self.! !

!OBRadioButtonBar methodsFor: 'constructing' stamp: 'cwp 11/27/2004 21:02'!
noteNewOwner: aMorph
	| window |
	window := aMorph containingWindow.
	window ifNotNil: [self adoptPaneColor: window paneColor]! !

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 2/25/2004 00:36'!
on: aModel list: listSelector selected: selectionGetter changeSelected: selectionSetter 
	self model: aModel.
	selection _ 0.
	getListSelector _ listSelector.
	getSelectionSelector _ selectionGetter.
	setSelectionSelector _ selectionSetter.
	self initGeometry.
	self updateList.! !

!OBRadioButtonBar methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 18:47'!
push: aButton
	| index |
	index _ buttons indexOf: aButton.
	model perform: setSelectionSelector with: index.! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 11/27/2004 17:58'!
updateButtons
	| labels |
	labels := model perform: getListSelector.
	buttons := Array new: labels size.
	labels withIndexDo: 
			[:label :index | 
			buttons at: index put: (OBButtonModel withLabel: label inBar: self)].
	selection := self getSelectionIndex! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 11/27/2004 17:58'!
updateList
	self 
		updateButtons; 
		updateMorphs! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 11/2/2004 01:09'!
updateMorphs
	self removeAllMorphs.
	buttons do: [:button | self addMorphBack: button morph].
	self bounds: (self bounds withHeight: submorphs first height)! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:39'!
updateSelection
	| oldSelection |
	oldSelection _ selection.
	selection _ self getSelectionIndex.
	self withButtonAt: oldSelection do: [:button | button selectionChanged].
	self withSelectedButtonDo: [:button | button selectionChanged]! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/25/2004 00:35'!
update: aSymbol
	aSymbol = getListSelector ifTrue: [self updateList. ^ self].
	aSymbol = getSelectionSelector ifTrue: [self updateSelection]! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:38'!
withButtonAt: index do: aBlock
	^ (buttons at: index ifAbsent: [nil]) ifNotNilDo: aBlock! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:38'!
withSelectedButtonDo: aBlock
	^ self withButtonAt: selection do: aBlock! !

Object subclass: #OBActor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBActor commentStamp: 'cwp 1/7/2005 23:14' prior: 0!
Actors provide the basic units of domain-related functionality. They are attached to metanodes, and supply the Actions used to build context menus and buttons in the browser. Actors can be proxies for functionality supplied by other classes, as in OBFileRegistryActor, or they may supply the funtionality them selves. 

OBActor is an abstract class which defines the protocol Actors must implement. Actors need not be subclasses of OBActor, however.!

!OBActor methodsFor: 'public' stamp: 'cwp 2/4/2004 15:25'!
actionsForNode: aNode
	^#()! !

!OBActor methodsFor: 'public' stamp: 'cwp 2/5/2004 20:12'!
actionsForParent: aNode
	^#()! !

!OBActor methodsFor: 'accessing' stamp: 'cwp 9/16/2004 23:25'!
actionWithLabel: aString selector: aSymbol 
	self shouldBeImplemented! !

OBActor subclass: #OBNodeActor
	instanceVariableNames: 'nodeClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Actors'!

!OBNodeActor commentStamp: 'cwp 3/5/2004 12:23' prior: 0!
For simplicity, many actions are implemented by the nodes themselves. When the contextual menu for a node is displayed, this Actor gathers those actions and makes them available in to the user.

If a node class is specified, the OBNodeActor will also make available any class side actions provided by the node class. This is convenient for the user, since it makes it possible to invoke an action without first selecting a node. Many 'create' actions are implemented in this way.

iVars:

nodeClass - if not nil, this class's class-side actions will be available to the user as well.!

!OBNodeActor class methodsFor: 'as yet unclassified' stamp: 'cwp 3/3/2004 22:51'!
onNodeClass: aClass
	^ self new nodeClass: aClass! !

!OBNodeActor methodsFor: 'public' stamp: 'cwp 1/23/2004 23:24'!
actionsForNode: aNode
	^ aNode actions! !

!OBNodeActor methodsFor: 'public' stamp: 'cwp 3/3/2004 22:46'!
actionsForParent: aNode
	^ nodeClass 
		ifNotNil: [nodeClass actionsForParent: aNode]
		ifNil: [#()]! !

!OBNodeActor methodsFor: 'public' stamp: 'cwp 3/3/2004 22:42'!
nodeClass: aClass
	nodeClass _ aClass! !

Object subclass: #OBBrowser
	instanceVariableNames: 'panels dispatcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBBrowser commentStamp: 'cwp 1/7/2005 23:17' prior: 0!
OBBrowser is the core of OmniBrower. It's the root object for each browser and the model for the SystemWindows which display them. Its main responsibily is managing panels, particularly passing update messages between them.

On the class side, OBBrowser provides some default settings for creating and opening browsers. Subclasses can override these settings to acheive customized behavior.

iVars:

panels	- A collection of objects which manage submorphs of the browser's SystemWindow.

cVars:

MetaGraphs - A dictionary matching names to metagraphs!

!OBBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:10'!
defaultMetaNode
	self subclassResponsibility! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/6/2004 23:20'!
defaultRootNode
	self subclassResponsibility! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:10'!
definitionPanel
	^ OBDefinitionPanel new! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:01'!
maxPanes
	^ self paneCount! !

!OBBrowser class methodsFor: 'metagraphs' stamp: 'cwp 11/7/2004 21:17'!
metagraphMissing: aSymbol
	^ self error: 'There is no metagraph named ', aSymbol.! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:12'!
metaNode: metaNode node: rootNode 
	^ self metaNode: metaNode root: rootNode selection: nil panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/17/2005 01:16'!
metaNode: metaNode root: rootNode selection: selectedNode
	^ self metaNode: metaNode root: rootNode selection: selectedNode panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/6/2004 21:59'!
metaNode: metaNode root: rootNode selection: selectedNode panels: panels
	| browser |
	browser _ self basicNew initialize.
	panels do: [:ea | browser addPanel: ea].
	browser setMetaNode: metaNode node: rootNode.
	selectedNode ifNotNil: [browser jumpTo: selectedNode].
	^ browser! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:00'!
minPanes
	^ self paneCount! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 16:55'!
navigationPanel
	^ OBColumnPanel minPanes: self minPanes maxPanes: self maxPanes! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/16/2005 23:25'!
new
	^ self 
		metaNode: self defaultMetaNode
		root: self defaultRootNode
		selection: nil
		panels: self panels! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 12/6/2004 23:22'!
open
	^ self new open! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 12/5/2004 17:03'!
openRoot: rootNode selection: selectedNode
	^ (self root: rootNode selection: selectedNode) open! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 11/26/2004 22:27'!
optionalButtonPanel
	| labels panel |
	labels _ self optionalButtons.
	(Preferences optionalButtons and: [labels isEmpty not]) ifTrue: 
		[panel _ OBFixedButtonPanel new.
		labels do: [:ea | panel addButtonWithLabel: ea]].
	^ panel! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:07'!
optionalButtons
	^ #()! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:01'!
paneCount
	^ 4! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 15:06'!
panels
	^ {self navigationPanel. self optionalButtonPanel. self definitionPanel} 
		reject: [:ea | ea isNil]! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 15:26'!
root: aNode 
	^ self root: aNode selection: nil! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/16/2005 23:25'!
root: rootNode selection: selectedNode
	^ self metaNode: self defaultMetaNode root: rootNode selection: selectedNode panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/6/2004 22:04'!
selection: selectedNode
	^ self root: self defaultRootNode selection: selectedNode! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:32'!
title
	^ 'OmniBrowser'! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 13:02'!
titleForRoot: aNode
	^ self title! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 9/17/2005 15:18'!
addPanel: aPanel
	panels add: aPanel.
	aPanel browser: self.
	self dispatcher addObserver: aPanel! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:00'!
currentNode
	^self navigationPanel currentNode! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:00'!
currentOrRootNode
	^self navigationPanel currentOrRootNode! !

!OBBrowser methodsFor: 'morphic' stamp: 'cwp 11/27/2004 22:21'!
defaultBackgroundColor
	^ Color gray veryMuchLighter! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 8/31/2004 13:04'!
defaultLabel
	^ self class titleForRoot: self root! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 9/17/2005 15:17'!
dispatcher
	^ dispatcher! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 9/17/2005 15:18'!
initialize
	panels := OrderedCollection new.
	dispatcher := OBDispatcher new.
	dispatcher addObserver: self! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
jumpToRoot
	^self navigationPanel jumpToRoot! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
jumpTo: aNode 
	self navigationPanel jumpTo: aNode! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 11/20/2004 21:05'!
labelString
	| label |
	label := self navigationPanel labelString.
	^label 
		ifNil: [self defaultLabel]
		ifNotNil: [self defaultLabel , ': ' , label]! !

!OBBrowser methodsFor: 'morphic' stamp: 'cwp 12/5/2004 14:23'!
morph
	| window |
	window := SystemWindow labelled: self defaultLabel.
	window model: self.
	panels isEmpty ifFalse: [self morphicPanelLayout addMorphsTo: window].
	^window! !

!OBBrowser methodsFor: 'morphic' stamp: 'cwp 11/21/2004 00:31'!
morphicPanelLayout
	^ OBMorphicPanelLayout for: panels! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 12/5/2004 16:25'!
navigationPanel
	^ panels 
		detect: [:ea | ea isNavigation] 
		ifNone: [self error: 'No navigation panel configured']! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 9/17/2005 17:35'!
nodeChanged
	self changed: #relabel.
! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 9/18/2005 12:10'!
okToChange
	^ self dispatcher okToChangeNode! !

!OBBrowser methodsFor: 'opening' stamp: 'cwp 10/17/2004 21:31'!
open
	^ OBBrowseRequest signal: self! !

!OBBrowser methodsFor: 'morphic' stamp: 'cwp 11/21/2004 00:30'!
openInMorphic
	^ self morph openInWorld! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:53'!
panels
	^ panels! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 9/17/2005 17:48'!
root
	^self navigationPanel root! !

!OBBrowser methodsFor: 'accessing' stamp: 'dvf 9/5/2005 17:30'!
selectionPath
	^self navigationPanel selectionPath! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 12/5/2004 16:24'!
setMetaNode: aMetaNode node: aNode 
	self navigationPanel setMetaNode: aMetaNode node: aNode! !

Object subclass: #OBButtonModel
	instanceVariableNames: 'bar label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBButtonModel commentStamp: 'cwp 3/5/2004 12:15' prior: 0!
An OBButtonModel serves as a model for PluggableButtonMorphs used OBRadioButtonBar. OBRadioButtonBar cannot be a direct model for its PBMs, since it can contain a variable number of buttons.

iVars:

bar		- the OBRadioButton bar to which this button belongs
label	- the label of the button!

!OBButtonModel class methodsFor: 'as yet unclassified' stamp: 'cwp 8/29/2004 13:31'!
offColor
	^ Color lightGray twiceLighter! !

!OBButtonModel class methodsFor: 'as yet unclassified' stamp: 'cwp 8/29/2004 13:30'!
onColor
	^ Color lightGray lighter! !

!OBButtonModel class methodsFor: 'as yet unclassified' stamp: 'cwp 2/24/2004 18:35'!
withLabel: aString inBar: aRadioButtonBar
	^ self new label: aString; bar: aRadioButtonBar! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 2/24/2004 18:29'!
bar: aRadioButtonBar
	bar _ aRadioButtonBar! !

!OBButtonModel methodsFor: 'testing' stamp: 'cwp 11/27/2004 19:09'!
isEnabled
	^ bar isEnabled: self! !

!OBButtonModel methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 18:42'!
isSelected
	^ bar isSelected: self! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 3/2/2004 21:46'!
label
	^ label! !

!OBButtonModel methodsFor: 'callbacks' stamp: 'avi 11/29/2004 21:56'!
labelMorph
	^ (StringMorph 
		contents: self label 
		font: Preferences standardDefaultTextFont)
			color: (self isEnabled ifTrue: [Color black] ifFalse: [Color gray]);
			yourself! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 2/24/2004 18:29'!
label: aString
	label _ aString! !

!OBButtonModel methodsFor: 'morphs' stamp: 'cwp 11/27/2004 19:06'!
morph
	| morph |
	morph _ PluggableButtonMorph
				on: self
				getState: #isSelected
				action: #push
				label: #labelMorph.
	morph 
		borderWidth: 2;
		borderRaised;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	^ morph! !

!OBButtonModel methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 19:30'!
push
	bar push: self.! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 11/27/2004 00:50'!
selectionChanged
	self changed: #isSelected.
	self changed: #labelMorph! !

Object subclass: #OBColumn
	instanceVariableNames: 'panel filter children selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBColumn commentStamp: 'cwp 1/7/2005 23:19' prior: 0!
An OBColumn manages a list of nodes, which it displays in a PluggableListMorph in the pane scroller at the top of the browser. All instances of OBColumn belong to an OBColumnPanel. It's main responsibility is keeping its list - and those of its neighbours - up to date. Each column has a MetaNode, which provides the list contents. It uses a filter to meditate between its self and the MetaNode.

iVars:

panel		- the panel which owns the column
filter		- the filter which manages the column's MetaNode.
parent		- the node selected in the column to the left of this column
children 	- the nodes which make up this column's list
selection 	- the index of the node selected by the user!

!OBColumn class methodsFor: 'instance creation' stamp: 'cwp 9/18/2005 13:26'!
inPanel: aBrowser
	^ self new setPanel: aBrowser! !

!OBColumn class methodsFor: 'instance creation' stamp: 'cwp 9/18/2005 13:25'!
inPanel: aPanel metaNode: aMetaNode node: aNode
	^ self new
			setPanel: aPanel
			metaNode: aMetaNode
			node: aNode! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 9/23/2005 22:14'!
acceptDroppingMorph: transferMorph event: evt inMorph: listMorph
	^ [(self nodeForDroppedMorph: transferMorph event: evt inMorph: listMorph)
		acceptDroppedNode: transferMorph passenger]
			on: OBDispatcherRequest
			do: [:notification | notification resume: self dispatcher]
! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 1/30/2004 00:38'!
accept: aText notifying: aController
	self selectedNode accept: aText notifying: aController.! !

!OBColumn methodsFor: 'actions' stamp: 'cwp 9/24/2005 22:23'!
actionSetsForParentNode
	^ self filter children gather: [:child | child actionSetsForParent: self parent].	! !

!OBColumn methodsFor: 'actions' stamp: 'cwp 9/14/2004 19:08'!
actionSetsForSelectedNode
	| node |
	node _ self selectedNode ifNil: [^ #()].
	^ node metaNode actionSetsForNode: node	! !

!OBColumn methodsFor: 'actions' stamp: 'hpt 5/17/2004 14:19'!
actionsForKeystroke: aChar
	^self actionsForParentNode, self actionsForSelectedNode
		select: [:anAction | anAction keystroke == aChar]! !

!OBColumn methodsFor: 'actions' stamp: 'cwp 9/24/2005 22:23'!
actionsForParentNode
	^ self filter children gather: [:child | child actionsForParent: self parent].	! !

!OBColumn methodsFor: 'actions' stamp: 'avi 3/6/2004 16:26'!
actionsForSelectedNode
	| node |
	node _ self selectedNode ifNil: [^ #()].
	^ node metaNode actionsForNode: node	! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:49'!
basicNext
	^panel columnAfter: self! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/17/2004 00:18'!
browser
	^ panel browser! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 12/14/2003 22:15'!
buttonHeight
	^ 20! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/19/2005 22:52'!
childrenChanged: aNode
	(self selectedNode = aNode) ifTrue:
		[self refresh]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/25/2005 23:56'!
clear
	filter _ nil.
	children _ #().
	self clearSelection.
	self changed: #list.
	self changed: #selection.
	self changed: #filter.! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 18:05'!
clearSelection
	selection _ 0! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/9/2004 21:06'!
columnWithHeader
	| col bh |
	col _ BorderedMorph new
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		clipSubmorphs: true;
		color: Color transparent;
		cellInset: 0;
		borderWidth: 0;
		layoutPolicy: ProportionalLayout new.
	
	bh _ self buttonHeight negated.
	col
		addMorph: self listMorph
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1) offsets: (0@0 corner: 0@bh)).
	col
		addMorph: self filter buttonMorph
		fullFrame: (LayoutFrame fractions: (0@1 corner: 1@1) offsets: (0@bh corner: 0@0)).
		

	^ col! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/24/2005 22:56'!
createNext
	| nextMetaNode created |
	nextMetaNode := self nextMetaNode.
	created := nextMetaNode columnInPanel: panel node: self selectedNode.
	panel pushColumn: created.
	^created! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/16/2004 21:52'!
definitionChanged: aDefinition 
	(self browser panels detect: [:ea | ea isDefinitionPanel] ifNone: [^self]) 
		definition: aDefinition! !

!OBColumn methodsFor: 'printing' stamp: 'avi 3/6/2004 16:23'!
descriptor
	self isEmpty ifTrue: [^ 'empty'].
	self hasSelection ifTrue: [^ self selectedNode name].
	^ ''! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 9/18/2005 09:27'!
dispatcher
	^ panel dispatcher! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 13:45'!
dragEnabled
	^ true! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:35'!
dragPassengerFor: item inMorph: listMorph
	^ self nodeForItem: item inMorph: listMorph! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:37'!
dragTransferType
	^ #OmniBrowser! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:36'!
dragTransferTypeForMorph: listMorph
	^ self dragTransferType! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:30'!
dropEnabled
	^ true! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:58'!
filter
	^ filter! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 18:47'!
filter: aFilter
	filter _ aFilter monitor: self.
	self changed: #filter.! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 9/24/2005 22:29'!
getChildren
	children _ self filter nodesForParent: self parent.
	^ children! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 23:07'!
hasSelection
	^ self selection > 0! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 12/20/2004 22:09'!
includesNode: aNode
	^ children includes: aNode! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 9/24/2005 22:29'!
isEmpty
	^ self parent isNil! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 9/24/2005 00:19'!
keystroke: aChar from: aMorph
	self isEmpty ifFalse: [(self actionsForKeystroke: aChar)
									do: [:action | 
										action dispatcher: self dispatcher.
										action trigger]]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 12/8/2004 21:21'!
list
	^ children 
		ifNil: [#()]
		ifNotNil: [children collect: [:ea | ea displayString]]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 12/8/2004 21:22'!
listAt: index
	^ (children at: index ifAbsent: [^ '']) displayString! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/18/2005 09:27'!
listChanged
	self refresh.
	self nextChanged.
	self dispatcher nodeChanged! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 11/27/2004 01:36'!
listMorph
	^ (PluggableListMorph
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:
		menu: #menu:
		keystroke: #keystroke:from:)
			getListElementSelector: #listAt:;
			getListSizeSelector: #listSize;
			dragEnabled: self dragEnabled;
			dropEnabled: self dropEnabled;
			borderWidth: 0;
			autoDeselect: false;
			adoptPaneColor: panel defaultBackgroundColor;
			yourself! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 2/29/2004 21:48'!
listSize
	^ children
		ifNil: [0]
		ifNotNil: [children size]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 9/24/2005 00:19'!
menu: aMenu
	self isEmpty ifFalse: [self actionSetsForParentNode, self actionSetsForSelectedNode
							do: [:set |set do: [:action | 
											action dispatcher: self dispatcher.
											action addItemToMenu: aMenu]]
							separatedBy: [aMenu addLine]].
	^ aMenu! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:59'!
metaNode
	^ self filter metaNode! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 9/26/2005 00:02'!
metaNode: aMetaNode parent: aNode
	self filter: aMetaNode filter.
	self getChildren.
	self clearSelection.
	self changed: #list.
	self nextChanged.
	self dispatcher nodeChanged.
	panel clearAfter: self! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/12/2004 19:17'!
morph
	^ self isEmpty 
		ifTrue: [self simplePane]
		ifFalse: [self filter wantsButton
							ifTrue: [self paneWithHeader]
							ifFalse: [self simplePane]]! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 20:50'!
next
	^ self basicNext
		ifNil: [(self hasSelection and: [self shouldBeLast not])
				ifTrue: [self createNext]]
	! !

!OBColumn methodsFor: 'updating' stamp: 'avi 3/6/2004 23:11'!
nextChanged
	self hasSelection ifTrue:
		[self next ifNotNilDo:
			[:next |
			next
				metaNode: self nextMetaNode 
				parent: self selectedNode]]! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:11'!
nextMetaNode
	^ self selectedNode metaNode! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/19/2005 22:58'!
nodeChanged: aNode
	(children includes: aNode) ifTrue: [self refresh]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/18/2005 14:24'!
nodeDeleted: aNode 
	"This gets called if an action causes the currently selected node to be deleted."

	self selectedNode = aNode
		ifTrue: 
			[self getChildren.
			self changed: #list.
			self selection: 0]! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 3/3/2004 02:23'!
nodeForDroppedMorph: transferMorph event: evt inMorph: pluggableListMorph
	| index item |
	index _ pluggableListMorph rowAtLocation: evt position.
	index = 0 ifTrue: [^ nil].
	item _ pluggableListMorph listMorph item: index.
	^ self nodeForItem: item inMorph: pluggableListMorph! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 22:18'!
nodeForItem: item inMorph: pluggableListMorph
	^ children 
		detect: [:child | child name = item contents]
		ifNone: [item contents]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 9/18/2005 12:10'!
okToChange
	^ self dispatcher okToChangeNode ! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:51'!
panel: aBrowser 
	panel := aBrowser! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/12/2004 18:11'!
paneWithHeader
	^ OBPane forColumn: self withFilter: self filter! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 9/24/2005 22:45'!
parent
	^ panel parentNodeForColumn: self! !

!OBColumn methodsFor: 'printing' stamp: 'cwp 2/11/2004 23:31'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self descriptor.
	aStream nextPut: $)! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/26/2005 00:02'!
refresh
	| node oldChildren |
	self isEmpty ifTrue: [^self].
	node := self selectedNode.
	oldChildren := children.
	self getChildren.
	children = oldChildren 
		ifFalse: 
			[self selectSilently: node.
			self hasSelection 
				ifFalse: 
					[panel clearAfter: self.
					self dispatcher nodeChanged]].
	self changed: #list! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 9/26/2005 00:14'!
selectAncestorOf: aNode
	| ancestor |
	ancestor _ self filter selectAncestorOf: aNode withParent: self parent.
	ancestor ifNotNil:
		[self 
			getChildren; 
			changed: #list;
			select: ancestor.
		panel selected: self].
	^ ancestor! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 2/2/2004 21:54'!
selectedNode
	^ children at: self selection ifAbsent: [].! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 11/5/2003 19:01'!
selection
	^ selection ifNil: [0]! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 9/26/2005 00:00'!
selectionChanged
	self hasSelection
		ifFalse: [panel clearAfter: self]
		ifTrue: [panel selected: self].
	self changed: #selection.
	self dispatcher nodeChanged! !

!OBColumn methodsFor: 'callbacks' stamp: 'avi 3/6/2004 16:22'!
selection: anInteger
	selection _ anInteger.
	self selectionChanged.
! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 12/20/2004 22:54'!
selectSilently: aNode
	selection _ children indexOf: (children 
										detect: [:ea | ea name = aNode name] 
										ifNone: [nil])! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 12/8/2004 22:10'!
select: aNode
	self selection: (children indexOf: (children detect: [:ea | ea = aNode] ifNone: []))! !

!OBColumn methodsFor: 'initializing' stamp: 'cwp 11/16/2004 21:57'!
setPanel: aPanel 
	panel := aPanel.
	children := #().
	self clearSelection! !

!OBColumn methodsFor: 'initializing' stamp: 'cwp 9/24/2005 22:59'!
setPanel: aPanel metaNode: aMetaNode node: aNode 
	panel := aPanel.
	filter := aMetaNode filter monitor: self.
	children _ self filter nodesForParent: aNode.
	self clearSelection! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 16:27'!
shouldBeLast
	^ self hasSelection not or: [self nextMetaNode hasChildren not]! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/12/2004 18:10'!
simplePane
	^ OBPane forColumn: self! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:18'!
wantsButton
	^ self filter notNil and: [self filter wantsButton]! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 21:26'!
wantsDroppedMorph: transferMorph event: evt inMorph: listMorph
	| node |
	(transferMorph isKindOf: TransferMorph) ifFalse: [^ false].
	transferMorph dragTransferType == self dragTransferType ifFalse: [^ false].
	node _ self nodeForDroppedMorph: transferMorph event: evt inMorph: listMorph.
	^ node notNil and: [node wantsDroppedNode: transferMorph passenger]! !

Object subclass: #OBDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBDefinition commentStamp: 'cwp 1/7/2005 23:38' prior: 0!
The responsibility of a definition is to express a node's composition textually and respond to changes in the text by updating the node it represents. For example,a file browser might use a file definition to allow editing of a files contents.!

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:51'!
accept: aText notifying: aController
	^ self text: aText! !

!OBDefinition methodsFor: 'accessing' stamp: 'cwp 3/23/2004 00:16'!
classRef
	^ nil! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:08'!
doItContext
	^ nil! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 2/28/2005 23:23'!
doItReceiver
	| class |
	class _ self selectedClass.
	^ class
		ifNotNil: [class theNonMetaClass]
		ifNil: [FakeClassPool new]! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 22:16'!
selection
	^ 1 to: 0! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:52'!
text
	^ ''! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 22:16'!
textSelection
	^ self selection! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:52'!
text: aText
	^ false! !

Object subclass: #OBDispatcher
	instanceVariableNames: 'observers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBDispatcher class methodsFor: 'as yet unclassified' stamp: 'cwp 9/23/2005 23:30'!
current
	^ OBDispatcherRequest signal 
		ifNil: 	[self halt.
				self new]
	! !

!OBDispatcher methodsFor: 'observers' stamp: 'cwp 9/17/2005 15:14'!
addObserver: anObserver
	observers addFirst: anObserver! !

!OBDispatcher methodsFor: 'forwarding' stamp: 'cwp 9/18/2005 12:08'!
doesNotUnderstand: aMessage
	^ self forward: aMessage! !

!OBDispatcher methodsFor: 'forwarding' stamp: 'cwp 9/18/2005 13:26'!
forward: aMessage
	^ observers
		select: [:ea | ea respondsTo: aMessage selector]
		thenCollect: [:ea | aMessage sendTo: ea].! !

!OBDispatcher methodsFor: 'initialize-release' stamp: 'cwp 9/17/2005 15:15'!
initialize
	observers _ OrderedCollection new.
	! !

!OBDispatcher methodsFor: 'accumulators' stamp: 'cwp 9/18/2005 12:09'!
okToChangeNode
	^ (self forward: (Message selector: #okToChangeNode))
		allSatisfy: [:ea | ea]! !

!OBDispatcher methodsFor: 'observers' stamp: 'cwp 9/6/2005 00:16'!
removeObserver: anObserver
	observers remove: anObserver! !

Object subclass: #OBFilter
	instanceVariableNames: 'metaNode monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBFilter commentStamp: 'cwp 3/4/2004 21:53' prior: 0!
A browser's metagraph defines the way in which the user may traverse the graph of objects which make up the browser's domain. But it's not always desirable to have all the nodes made available by the metagraph to be visible in the browser. An filter provides a strategy for filtering out some of the nodes from the display. 

OBFilter provides a "null" filter - one that does no filtering at all - and serves as a superclass for other filters. !

!OBFilter class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 20:54'!
forMetaNode: aMetaNode
	^ self new setMetaNode: aMetaNode! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/9/2004 20:50'!
children
	^ metaNode children! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/11/2004 23:48'!
metaNode
	^ metaNode! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:56'!
monitor: aMonitor
	monitor _ aMonitor! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/9/2004 20:52'!
nodesForParent: aNode
	^ metaNode nodesForParent: aNode! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/29/2004 14:34'!
selectAncestorOf: aNode withParent: parentNode
	^ (metaNode nodesForParent: parentNode)
		detect: [:child | child isAncestorOf: aNode] 
		ifNone: [nil]! !

!OBFilter methodsFor: 'initalizing' stamp: 'cwp 2/9/2004 20:54'!
setMetaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:06'!
wantsButton
	^ false! !

OBFilter subclass: #OBModalFilter
	instanceVariableNames: 'selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBModalFilter commentStamp: 'cwp 1/7/2005 23:39' prior: 0!
OBModalFilter is used to implement the 'instance/?/class' buttons in a standard class browser. In functional terms it filters the nodes of a column according to the edge of the metagraph that they correspond to. OBModalFilter displays an OBRadioButtonBar in its column's pane, with one button per edge. Only nodes from the currently selected edge are allowed in the column.

iVars:

selection	- the currently selected edge!

!OBModalFilter methodsFor: 'morphs' stamp: 'cwp 11/27/2004 18:01'!
buttonMorph
	^ OBRadioButtonBar
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:! !

!OBModalFilter methodsFor: 'selection' stamp: 'cwp 2/9/2004 21:11'!
incrementSelection
	selection _ selection + 1.
	(selection > metaNode edges size) ifTrue: [selection _ 1].! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/25/2004 00:28'!
list
	^ metaNode edges collect: [:edge | edge label]! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 3/3/2004 00:23'!
nodesForParent: aNode
	^ self selectedEdge nodesForParent: aNode! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 2/29/2004 14:35'!
selectAncestorOf: aNode withParent: parentNode
	| ancestor |
	1 	to: metaNode edges size 
	  	do: [:i | selection _ i.
			ancestor _ (self nodesForParent: parentNode)
				detect: [:child | child isAncestorOf: aNode] 
				ifNone: [nil].
			ancestor ifNotNil: [self changed: #selection. ^ ancestor]].
	^ nil! !

!OBModalFilter methodsFor: 'selection' stamp: 'cwp 2/9/2004 21:11'!
selectedEdge
	^ metaNode edges at: self selection ! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/9/2004 21:10'!
selection
	^ selection ifNil: [selection _ 1]! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/25/2004 00:33'!
selection: anInteger
	selection _ anInteger.
	self changed: #selection.
	monitor listChanged.
! !

!OBModalFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:08'!
wantsButton
	^ true! !

Object subclass: #OBMetaEdge
	instanceVariableNames: 'label selector metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaEdge commentStamp: 'cwp 1/7/2005 23:20' prior: 0!
An OBMetaEdge is an edge in the browser's metagraph. It represents a message sent to a node to obtain further nodes. It is refered to by the "parent" metanode, and refers to the "child" metanode.

iVars:

label 		-  a string describing the metaNode, for filters which allow the user 
			   to choose which edges to follow
selector	- when a node is selected by the user, this message will be 
			  sent to it to obtain its children
metaNode 	- a MetaNode corresponding to the nodes answered by the above message
		!

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:35'!
label: aString selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 22:32'!
selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aSelector asString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
label
	^ label! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
metaNode
	^ metaNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 3/3/2004 00:12'!
nodesForParent: aNode
	^ (aNode perform: selector) do: [:ea | ea metaNode: metaNode]
! !

!OBMetaEdge methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:03'!
printOn: aStream
	aStream nextPutAll: selector printString, '->'. 
	metaNode shortPrintOn: aStream.! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
selector
	^ selector! !

!OBMetaEdge methodsFor: 'initializing' stamp: 'cwp 2/7/2004 22:36'!
setLabel: aString selector: aSelector metaNode: aMetaNode
	label _ aString.
	selector _ aSelector.
	metaNode _ aMetaNode! !

Object subclass: #OBMetaNode
	instanceVariableNames: 'name filterClass columnClass edges actors displaySelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaNode commentStamp: 'cwp 1/7/2005 23:23' prior: 0!
A MetaNode represents a hypothetical node in the browser's domain graph.

iVars:

filterClass			- the class used to filter this hypothetical node's children
columnClass		- the class of column used to display this node's children
edges				- messages that could be sent to the node to obtain children
actors				- providers of functionality for manipulating the node
displaySelector		- the message used to retreive a nodes display name!

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/6/2004 21:06'!
named: aString
	^ self new name: aString! !

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/3/2004 23:08'!
new
	^ self basicNew initialize! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 9/14/2004 18:59'!
actionSetsForNode: aNode
	^ actors collect: [:actor | actor actionsForNode: aNode]! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 9/14/2004 19:01'!
actionSetsForParent: aNode
	^ actors collect: [:actor | actor actionsForParent: aNode]! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 9/14/2004 19:00'!
actionsForNode: aNode
	^ (self actionSetsForNode: aNode) gather: [:ea | ea]! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 9/14/2004 19:02'!
actionsForParent: aNode
	^ (self actionSetsForParent: aNode) gather: [:ea | ea]! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 3/3/2004 22:48'!
addActor: anActor
	actors add: anActor.! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/9/2004 20:28'!
childAt: aSelector labeled: aString put: aMetaNode
	edges add: (OBMetaEdge label: aString selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:35'!
childAt: aSelector put: aMetaNode
	edges add: (OBMetaEdge selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:44'!
children
	^ edges collect: [:edge | edge metaNode]! !

!OBMetaNode methodsFor: 'columns' stamp: 'avi 3/4/2004 03:11'!
columnClass: aClass
	columnClass _ aClass! !

!OBMetaNode methodsFor: 'columns' stamp: 'cwp 11/16/2004 22:03'!
columnInPanel: aBrowser node: aNode
	columnClass ifNil: [columnClass _ OBColumn].
	^ columnClass
		inPanel: aBrowser
		metaNode: self
		node: aNode! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 12/8/2004 21:41'!
displaySelector
	^ displaySelector ifNil: [displaySelector _ #name]! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 12/8/2004 21:42'!
displaySelector: aSelector
	displaySelector _ aSelector! !

!OBMetaNode methodsFor: 'metagraph' stamp: 'cwp 2/9/2004 21:12'!
edges
	^ edges! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:26'!
filter
	filterClass ifNil: [filterClass _ OBFilter].
	^ filterClass forMetaNode: self! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:02'!
filterClass: aClass
	filterClass _ aClass! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:38'!
hasChildren
	^ edges isEmpty not! !

!OBMetaNode methodsFor: 'initializing' stamp: 'cwp 12/8/2004 20:06'!
initialize
	edges _ OrderedCollection new.
	actors _ OrderedCollection new.! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:00'!
metaNode
	^ self! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:54'!
name
	^ name ifNil: ['MetaNode']! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:05'!
name: aString
	name _ aString! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:22'!
nodesForParent: aNode
	^ edges gather: [:edge | edge nodesForParent: aNode]! !

!OBMetaNode methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:01'!
printOn: aStream
	aStream nextPutAll: self name;cr.
	edges do: [:e | e printOn: aStream. aStream cr].! !

!OBMetaNode methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:01'!
shortPrintOn: aStream
	aStream nextPutAll: self name.! !

Object subclass: #OBMorphicPanelLayout
	instanceVariableNames: 'panels'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBMorphicPanelLayout commentStamp: 'cwp 1/7/2005 23:42' prior: 0!
OBMorphicPanelLayout implements a somewhat hairy algorithm for layoung out panels in a morphic SystemWindow. It is the default layout strategy used by OBBrowser. Subclasses of OBBrowser may use other layout strategies to customize their appearance.!

!OBMorphicPanelLayout class methodsFor: 'intance creation' stamp: 'cwp 11/20/2004 23:32'!
for: aCollection
	^ self new setPanels: aCollection! !

!OBMorphicPanelLayout methodsFor: 'public' stamp: 'cwp 11/21/2004 00:26'!
addMorphsTo: aMorph
	self addNavMorphTo: aMorph.
	(panels size > 1) ifTrue: [self addOtherMorphsTo: aMorph].
	! !

!OBMorphicPanelLayout methodsFor: 'private' stamp: 'cwp 11/21/2004 00:25'!
addNavMorphTo: aMorph
	aMorph 
		addMorph: self navigationPanel buildScroller
		fullFrame: (LayoutFrame fractions: (0 @ 0 extent: 1 @ self columnProportion)).
! !

!OBMorphicPanelLayout methodsFor: 'private' stamp: 'cwp 11/21/2004 00:37'!
addOtherMorphsTo: aMorph
	| heights variableCount fraction offset vExtent h nonNavPanels |
	nonNavPanels := panels reject: [:ea | ea isNavigation].
	nonNavPanels isEmpty ifTrue: [^ self].
	heights _ nonNavPanels collect: [:ea | ea morphHeight].
	variableCount _ heights 
						inject: 0 
						into: [:count :ea | ea = 0 ifTrue: [count + 1] ifFalse: [count]].
	fraction _ self columnProportion.
	offset _ 0.
	vExtent _  1- fraction / variableCount.
	nonNavPanels with: heights do:  [:panel :height | 
		h _ height = 0 ifTrue: [vExtent] ifFalse: [0].
		aMorph 
			addMorph: panel morph
			fullFrame: (LayoutFrame
							fractions: (0@fraction extent: 1@h)
							offsets: (0@offset corner: 0@height)).
		height = 0 
			ifTrue: 	[offset _ 0.
					fraction _ fraction + vExtent]
			ifFalse: 	[offset _ offset + height]]
						
		
						! !

!OBMorphicPanelLayout methodsFor: 'private' stamp: 'cwp 11/21/2004 00:26'!
columnProportion
	^ panels size = 1
		ifTrue: [1]
		ifFalse: [0.4]! !

!OBMorphicPanelLayout methodsFor: 'private' stamp: 'cwp 11/21/2004 00:28'!
navigationPanel
	^ panels detect: [:ea | ea isNavigation]! !

!OBMorphicPanelLayout methodsFor: 'initializing' stamp: 'cwp 11/21/2004 00:23'!
setPanels: aCollection 
	panels := aCollection! !

Object subclass: #OBNode
	instanceVariableNames: 'metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBNode commentStamp: 'cwp 3/4/2004 22:20' prior: 0!
A node is a wrapper for an object in the browser's domain graph. OBNode is an abstract superclass for concrete nodes which might appear in the browser.

iVars:

metaNode - the MetaNode which produced this node!

OBNode subclass: #OBCollectionNode
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBCollectionNode commentStamp: 'cwp 1/7/2005 23:31' prior: 0!
OBCollectionNode is a trivial wrapper for a collection of nodes. It is typically used as an artificial root node for metagraphs that have no natural root.!

!OBCollectionNode class methodsFor: 'as yet unclassified' stamp: 'cwp 3/15/2004 23:17'!
on: aCollection
	^ self new setCollection: aCollection! !

!OBCollectionNode methodsFor: 'navigating' stamp: 'cwp 3/15/2004 23:19'!
children
	^ collection! !

!OBCollectionNode methodsFor: 'displaying' stamp: 'cwp 3/15/2004 23:19'!
name
	^ collection species name! !

!OBCollectionNode methodsFor: 'initalizing' stamp: 'cwp 3/15/2004 23:18'!
setCollection: aCollection
	collection _ aCollection! !

OBNode subclass: #OBFakeNode
	instanceVariableNames: 'item parent children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFakeNode class methodsFor: 'browsing' stamp: 'cwp 11/7/2004 21:26'!
metagraph
	| fake |
	fake _ OBMetaNode new.
	fake 
		childAt: #children put: fake;
		addActor: (OBNodeActor onNodeClass: OBFakeNode).
	^ fake! !

!OBFakeNode class methodsFor: 'instance creation' stamp: 'cwp 12/15/2003 22:06'!
parent: aFakeNode item: aString
	^ self new parent: aFakeNode; item: aString! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 16:49'!
actions
	^ Array 
		with: (self action: #beZ withLabel: 'become z')
		with: (self action: #delete withLabel: 'delete')
		with: (self action: #deleteRoot withLabel: 'delete root')! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 2/25/2004 22:39'!
adopt: aNode
	aNode parent removeChild: aNode.
	aNode parent: self.
	children _ self children copyWith: aNode.
	self signalSelection! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 9/18/2005 18:56'!
beZ
	item _ 'z'.
	self signalChanged! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 1/23/2004 21:42'!
children
	^ children 
		ifNil: [children _ (self item endsWith: 'c')
					ifTrue: [Array new]
					ifFalse: [#($a $b $c) collect: [:i | OBFakeNode parent: self item: i asString]]]! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 17:12'!
delete
	parent ifNotNil: [parent removeChild: self].
	self signalDeletion! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 16:58'!
deleteRoot
	| node |
	node _ self parent.
	[node parent notNil] whileTrue: [node _ node parent].
	node signalDeletion.! !

!OBFakeNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:41'!
dropOnFakeNode: aNode
	aNode adopt: self! !

!OBFakeNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:40'!
dropSelector
	^ #dropOnFakeNode:! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 12/5/2004 16:16'!
isAncestorOf: aNode
	| ancestor |
	aNode name = self name ifTrue: [^ true].
	ancestor _ aNode.
	[ancestor isNil]
		whileFalse: [ancestor == self ifTrue: [^ true].
					ancestor _ ancestor parent].
	^ false! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/7/2003 19:18'!
item
	^ item ifNil: [item _ '']! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/7/2003 19:09'!
item: aSymbol
	item _ aSymbol! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 12/15/2003 22:05'!
name
	^ self parentName, self item ! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 2/25/2004 21:49'!
parent
	^ parent! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/15/2003 22:04'!
parentName
	^ (parent ifNil: [''] ifNotNil: [parent name])! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/15/2003 22:01'!
parent: aFakeNode
	parent _ aFakeNode! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 2/25/2004 21:53'!
removeChild: aFakeNode
	children _ self children copyWithout: aFakeNode! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 12/15/2003 22:03'!
text
	^ self name! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 9/18/2005 14:58'!
text: aText
	self item:( aText asString allButFirst: (self parentName size)).
	self demandSelection.
	^ true! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/5/2004 20:25'!
actionsForParent: aNode
	^ #()! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 22:11'!
fromAssociation: anAssociationNode
	^ self on: anAssociationNode value! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:28'!
acceptDroppedNode: aNode
 	^ aNode perform: self dropSelector with: self
! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 13:49'!
accept: aText notifying: aController
	^ self text: aText! !

!OBNode methodsFor: 'public' stamp: 'cwp 1/7/2004 09:25'!
actions
	^ #()! !

!OBNode methodsFor: 'utility' stamp: 'cwp 3/10/2004 00:29'!
action: aSelector buttonLabel: buttonString menuLabel: menuString
	^ OBAction
		label: menuString
		buttonLabel: buttonString
		receiver: self
		selector: aSelector
		arguments: #()! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/3/2004 22:29'!
action: aSelector withLabel: aString
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()! !

!OBNode methodsFor: 'utility' stamp: 'hpt 5/17/2004 16:51'!
action: aSelector withLabel: aString withIcon: anIcon
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()
		icon: anIcon! !

!OBNode methodsFor: 'utility' stamp: 'hpt 5/17/2004 14:28'!
action: aSelector withLabel: aString withKeystroke: aChar
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()
		keystroke: aChar! !

!OBNode methodsFor: 'utility' stamp: 'hpt 5/17/2004 16:51'!
action: aSelector withLabel: aString withKeystroke: aChar withIcon: anIcon
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()
		keystroke: aChar
		icon: anIcon! !

!OBNode methodsFor: 'utility' stamp: 'cwp 12/5/2004 20:11'!
action: aSelector withMenuLabel: aString withButtonLabel: aString2 withKeystroke: aChar withIcon: anIcon 
	^OBAction 
		label: aString
		buttonLabel: aString2
		receiver: self
		selector: aSelector
		arguments: #()
		keystroke: aChar
		icon: anIcon! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
ancestrySelector
	^ #isDescendantOf: ! !

!OBNode methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:00'!
asDraggableMorph
	^(StringMorph contents: self name)
		color: Color white;
		yourself! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/22/2004 20:48'!
definition
	^ self! !

!OBNode methodsFor: 'notification' stamp: 'cwp 9/23/2005 23:25'!
demandSelection
	^ OBDispatcher current selectNode: self! !

!OBNode methodsFor: 'notification' stamp: 'cwp 9/18/2005 14:03'!
dispatchDuring: aBlock
	OBDispatcherRequest signal ifNotNilDo: aBlock! !

!OBNode methodsFor: 'public' stamp: 'dvf 8/17/2005 13:57'!
displayName
	"expected to return a decorated text (rather than string) version of name"
	^self name asText! !

!OBNode methodsFor: 'displaying' stamp: 'cwp 12/8/2004 21:36'!
displayString
	^ self perform: metaNode displaySelector! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:29'!
dropSelector
	"Override in subclasses"
	
	^ #dropOnNode: ! !

!OBNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:57'!
hash
	^ self name hash! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:47'!
isAncestorOf: aNode
	^ aNode perform: self ancestrySelector with: self! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
isDescendantOf: aNode
	^ false! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode
	^ metaNode! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/10/2003 22:27'!
name
	self subclassResponsibility! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/7/2004 21:29'!
referenceForMethod: selector ofClass: className
	| classIsMeta symbol |
	classIsMeta _ className endsWith: ' class'.
	symbol _ classIsMeta ifTrue: [(className allButLast: 6) asSymbol] ifFalse: [className].
	^ MethodReference new 
		setClassSymbol: symbol
		classIsMeta: classIsMeta
		methodSymbol: selector
		stringVersion: symbol, '>>', selector! !

!OBNode methodsFor: 'notification' stamp: 'cwp 9/23/2005 21:47'!
signalChanged
	OBDispatcher current nodeChanged: self! !

!OBNode methodsFor: 'notification' stamp: 'cwp 9/23/2005 21:47'!
signalChildrenChanged
	OBDispatcher current childrenChanged: self! !

!OBNode methodsFor: 'notification' stamp: 'cwp 9/18/2005 14:03'!
signalDeletion
	self dispatchDuring: [:dispatcher | dispatcher nodeDeleted: self]! !

!OBNode methodsFor: 'notification' stamp: 'cwp 9/18/2005 14:51'!
signalSelection
	^ self dispatchDuring: [:dispatcher | dispatcher selectNodeCautiously: self]! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/7/2003 19:27'!
text
	^ ''! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 18:30'!
textSelection
	^ 1 to: 0! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/14/2004 14:09'!
text: aText
	^ false! !

!OBNode methodsFor: 'displaying' stamp: 'cwp 8/28/2004 00:14'!
title
	^ nil! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:28'!
wantsDroppedNode: aNode
	^ aNode respondsTo: self dropSelector! !

!OBNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:57'!
= other
	^ self class = other class and: [self name = other name]! !

Object subclass: #OBPanel
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBPanel commentStamp: 'cwp 1/7/2005 23:23' prior: 0!
A panel is an object that manages part of the browser's window. It provides a protocol for receiving notifications when the current domain node changes, and reacts to these changes.!

OBPanel subclass: #OBColumnPanel
	instanceVariableNames: 'root columns minPanes maxPanes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBColumnPanel commentStamp: 'cwp 12/6/2004 23:50' prior: 0!
OBColumnPanel handles navigation around the nodes of the object graph. It maintains a list of columns, which track the user's path through the node tree. As nodes are selected, additional columns are added to the list, which appear as panes on the right of the panel. 

iVars:

columns	- A collection of OBColumns, each of which manages a single pane in the scroller.
minPanes - The minimum number of panes that should ever be visible.
maxPanes - The maximum number of panes that should ever be visible.
!

!OBColumnPanel class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 16:56'!
minPanes: min maxPanes: max
	^ self basicNew setMinPanes: min maxPanes: max! !

!OBColumnPanel class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 16:56'!
new
	^ self minPanes: 1 maxPanes: 1! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 11/17/2004 23:24'!
buildScroller
	^ (OBPaneScroller withModel: self)
		name: 'scroller';
		yourself! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 9/25/2005 23:58'!
clearAfter: aColumn 
	| start |
	start _ (columns indexOf: aColumn) + 1.
	start to: columns size do: [:i | (columns at: i) clear]! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 9/24/2005 22:04'!
columnAfter: aColumn 
	^ self columns after: aColumn ifAbsent: [nil]


! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 9/24/2005 22:05'!
columnBefore: aColumn 
	^ self columns before: aColumn ifAbsent: [nil]


! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:46'!
columns
	^columns! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:46'!
columns: anObject
	columns := anObject! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/16/2004 22:22'!
currentColumn
	^self columns reversed detect: [:ea | ea hasSelection] ifNone: []! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:22'!
currentNode
	^ self currentColumn ifNotNilDo: [:column | column selectedNode]! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:23'!
currentOrRootNode
	^(self columns reversed detect: [:ea | ea hasSelection]
		ifNone: [^self columns first parent]) selectedNode! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/26/2004 21:55'!
defaultBackgroundColor
	^ browser defaultBackgroundColor! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/16/2004 23:01'!
emptyColumn
	^ OBColumn inPanel: self! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 1/8/2005 14:20'!
hopTo: aNode 
	| column |
	column := self columns last.
	[column refresh; includesNode: aNode]
		whileFalse: [column := columns 
									before: column 
									ifAbsent: [^ self jumpTo: aNode]].
	column select: aNode! !

!OBColumnPanel methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:10'!
isNavigation
	^ true! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 11/16/2004 23:34'!
jumpToRoot
	^ self columns first selection: 0! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 1/8/2005 18:09'!
jumpTo: aNode 
	| column ancestor |
	column := self columns first.
	
	[ancestor := column selectAncestorOf: aNode.
	ancestor = aNode or: [ancestor isNil]] 
			whileFalse: [column := self columns 
											after: column
											ifAbsent: [^ self]]! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 23:51'!
labelString
	| label |
	self columns reversed do: 
		[:ea | 
		label := ea selectedNode ifNotNilDo: [:node | node title].
		label ifNotNil: [^ label]].
	^ nil
! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 23:29'!
maxPanes
	^maxPanes! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:54'!
minPanes
	^ minPanes! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 11/20/2004 22:22'!
morph
	^ (OBPaneScroller withModel: self)
		name: 'scroller';
		yourself! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 11/20/2004 22:26'!
morphHeight
	^ 0! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 9/18/2005 09:28'!
nodeDeleted: aNode
	aNode = self root 
		ifTrue: 
			[self columns first clear.
			self dispatcher nodeChanged]! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/23/2004 00:52'!
okToReclaimPane
	columns size > minPanes ifFalse: [^ false].
	^ columns last isEmpty or: [(columns at: columns size - 1) shouldBeLast].
! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/17/2004 22:48'!
panes
	^ columns collect: [:ea | ea morph]! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 9/24/2005 22:44'!
parentNodeForColumn: aColumn
	^ (columns before: aColumn ifAbsent: [^ root])
		selectedNode! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 9/18/2005 11:51'!
popColumn
	self dispatcher removeObserver: self columns removeLast.
! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 9/18/2005 11:50'!
pushColumn: aColumn 
	self columns addLast: aColumn.
	self changed: #panes.
	self dispatcher addObserver: aColumn.
! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 9/18/2005 11:49'!
reclaimPanes
	| old |
	old := columns size.
	[self okToReclaimPane] whileTrue: [self popColumn].
	^ old - columns size! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 9/24/2005 22:47'!
root
	^ root! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 9/25/2005 23:45'!
selected: aColumn 
	aColumn next ifNotNilDo: 
		[:next | 
		next 
			metaNode: aColumn nextMetaNode 
			parent: aColumn selectedNode].! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'dvf 9/5/2005 18:08'!
selectionPath
	^(self columns 
		collect: [:e | e parent] 
		thenSelect: [:e | e notNil]) allButFirst! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 9/18/2005 14:50'!
selectNodeCautiously: aNode
	^ self dispatcher okToChangeNode ifTrue: [self hopTo: aNode]! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 9/23/2005 23:28'!
selectNode: aNode
	^ self hopTo: aNode! !

!OBColumnPanel methodsFor: 'initializing' stamp: 'cwp 9/24/2005 22:45'!
setMetaNode: aMetaNode node: aNode 
	root _ aNode.
	root metaNode: aMetaNode.
	self pushColumn: (aMetaNode columnInPanel: self node: root).
	minPanes - self columns size 
		timesRepeat: [self pushColumn: self emptyColumn]! !

!OBColumnPanel methodsFor: 'initializing' stamp: 'cwp 12/5/2004 16:54'!
setMinPanes: min maxPanes: max
	columns _ OrderedCollection new.
	minPanes _ min.
	maxPanes _ max! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/18/2004 00:00'!
sizing
	^ (columns size max: minPanes) min: maxPanes! !

OBPanel subclass: #OBDefinitionPanel
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBDefinitionPanel commentStamp: 'cwp 1/7/2005 23:35' prior: 0!
OBDefinition serves as the model for the text pane of a typical browser. It's main responsibility is to act as a relay between a PluggableTextMorph and a Definition supplied by the currently selected node.

iVars:

browser - The browser of which this panel is a part.
!

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 9/23/2005 23:34'!
accept: aText notifying: aController
	^ self 
		withDefinitionDo: [:def | [def accept: aText notifying: aController]
									on: OBDispatcherRequest
									do: [:notification | notification resume: self dispatcher]]
		ifNil: [true]! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:56'!
definition: aDefinition
	definition _ aDefinition.
	self changed: #text! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:23'!
doItContext
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #doItContext) ifTrue: [def doItContext]]
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:23'!
doItReceiver
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #doItReceiver) ifTrue: [def doItReceiver]]
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 3/22/2004 20:06'!
getDefinition
	^ browser currentNode ifNotNilDo: [:node | node definition]! !

!OBDefinitionPanel methodsFor: 'testing' stamp: 'cwp 9/16/2004 15:22'!
isDefinitionPanel
	^ true! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 3/14/2004 15:02'!
isDirty
	^ self canDiscardEdits 
	! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 10/24/2004 09:16'!
menu: aMenu shifted: aBoolean
	| items |
	items _  aBoolean 
		ifTrue: [self shiftedYellowButtonMenu] 
		ifFalse: [self yellowButtonMenu].
	items do: [:ea |
			  ea = #-
				ifFalse: [aMenu add: ea first action: ea second]
				ifTrue: [aMenu addLine]].
	^ aMenu ! !

!OBDefinitionPanel methodsFor: 'constructing' stamp: 'cwp 10/30/2004 22:03'!
morph
	^ (OBPluggableTextMorph
		on: self
		text: #text
		accept: #accept:notifying:
		readSelection: #selection
		menu: #menu:shifted:)
			font: Preferences standardCodeFont;
			yourself
			
	"see CodeHolder>>buildMorphicCodePaneWith:"! !

!OBDefinitionPanel methodsFor: 'constructing' stamp: 'cwp 3/9/2004 21:35'!
morphHeight
	^ 0! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 9/17/2004 00:56'!
nodeChanged
	self definition: self getDefinition! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 9/18/2005 12:02'!
okToChangeNode
	| ok |
	self canDiscardEdits ifTrue: [^ true].
	ok _ OBConfirmationRequest
			prompt: 'Changes have not been saved.
Is it OK to discard those changes?'
			confirm: 'Discard changes'.
	ok ifTrue: [self changed: #clearUserEdits].
	^  ok.! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 10/24/2004 09:27'!
perform: aSelector orSendTo: anObject
	| receiver |
	receiver  _ (self respondsTo: aSelector) ifTrue: [self] ifFalse: [anObject].
	receiver perform: aSelector! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 12/5/2004 15:32'!
refresh
	| oldDefinition |
	definition ifNil: [^ self].
	oldDefinition _ definition.
	definition _ self getDefinition.
	definition text = oldDefinition text ifTrue: [^ self].
	self canDiscardEdits
			ifTrue: [self changed: #text]
			ifFalse: [self changed: #codeChangedElsewhere]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:22'!
selectedClass
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #selectedClass) ifTrue: [def selectedClass]] 
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 21:15'!
selection
	^ self withDefinitionDo: [:def | def textSelection] ifNil: [1 to: 0]! !

!OBDefinitionPanel methodsFor: 'constructing' stamp: 'cwp 10/24/2004 23:28'!
shiftedYellowButtonMenu
	^ {
		{'explain' translated.						#explain}.
		{'recognizer (r)' translated.					#recognizeCharacters}.
		{'verify spelling of word' translated.		#verifyWordSpelling}.
		#-.
		{'selectors containing it (W)' translated.		#methodNamesContainingIt}.
		{'method strings with it (E)' translated.		#methodStringsContainingit}.
		{'method source with it' translated.			#methodSourceContainingIt}.
		{'class names containing it' translated.		#classNamesContainingIt}.
		{'class comments with it' translated.			#classCommentsContainingIt}.
		{'change sets with it' translated.				#browseChangeSetsWithSelector}.
		#-.
		{'save contents to file...' translated.			#saveContentsInFile}.
		#-.
		{'more...' translated.							#yellowButtonActivity}.
	}
! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:42'!
text
	^ self withDefinitionDo: [:def | def text] ifNil: ['']! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 3/22/2004 20:41'!
withDefinitionDo: workBlock ifNil: nilBlock
	definition ifNil: [ ^ nilBlock value].
	^ workBlock value: definition! !

!OBDefinitionPanel methodsFor: 'constructing' stamp: 'cwp 10/24/2004 23:28'!
yellowButtonMenu
	^ {
		{'find...(f)' translated.				#find}.			     
		{'find again (g)' translated.			#findAgain}.		     
		{'set search string (h)' translated.	#setSearchString}.
		#-.	     
		{'do again (j)' translated.			#again}.		     
		{'undo (z)' translated.				#undo}.			     
		#-.	     
		{'copy (c)' translated.				#copySelection}.	     
		{'cut (x)' translated.					#cut}.			     
		{'paste (v)' translated.				#paste}.		     
		{'paste...' translated.					#pasteRecent}.		     
		#-.	     
		{'accept (s)' translated.				#accept}.		     
		{'cancel (l)' translated.				#cancel}.		     
		#-.	     
		{'do it (d)' translated.				#doIt}.			     
		{'print it (p)' translated.				#printIt}.		     
		{'inspect it (i)' translated.			#inspectIt}.		     
		{'explore it (I)' translated.			#exploreIt}.		     
		{'debug it' translated.				#debugIt}.		     
		#-.	     
		{'browse it (b)' translated.					#browseIt}.
		{'senders of it (n)' translated.				#sendersOfIt}.
		{'implementors of it (m)' translated.			#implementorsOfIt}.
		{'references to it (N)' translated.				#referencesToIt}.
		#-.
		{'more...' translated.					#shiftedTextPaneMenuRequest}.
	}
! !

OBPanel subclass: #OBFixedButtonPanel
	instanceVariableNames: 'models actions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBFixedButtonPanel commentStamp: 'cwp 1/7/2005 23:35' prior: 0!
OBFixedButtonPanel displays a horizontal row of buttons. In contrast to OBVarButtonPanel, the buttons do not change as nodes are selected in the navigation panel; instead they are enabled and disabled according to whether the actions they represent are applicable to the selected node.!

!OBFixedButtonPanel methodsFor: 'public' stamp: 'cwp 11/26/2004 22:52'!
addButtonWithLabel: label
	| model |
	model _ OBButtonModel withLabel: label inBar: self.
	models add: model.
	actions at: model put: nil! !

!OBFixedButtonPanel methodsFor: 'accessing' stamp: 'cwp 11/26/2004 22:53'!
buttonModels
	^ models! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/26/2004 22:40'!
createMorph
	^(RectangleMorph new)
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 0;
		wrapCentering: #center;
		cellPositioning: #leftCenter;
		rubberBandCells: true;
		yourself! !

!OBFixedButtonPanel methodsFor: 'initializing' stamp: 'cwp 11/26/2004 22:53'!
initialize
	models _ OrderedCollection new.
	actions _ Dictionary new.! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 11/27/2004 19:20'!
isEnabled: aModel
	^ (actions at: aModel) notNil! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 11/27/2004 19:20'!
isSelected: aModel
	^ true! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/27/2004 21:24'!
morph
	| morph |
	morph := self createMorph.
	self buttonModels do: [:ea | morph addMorphBack: (self morphForModel: ea)].
	^morph! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/27/2004 21:25'!
morphForModel: aModel
	| morph paneColor |
	morph _ aModel morph.
	paneColor _ browser defaultBackgroundColor.
	morph onColor: paneColor darker offColor: paneColor lighter.
	^ morph! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/26/2004 22:41'!
morphHeight
	^ 25! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 8/29/2004 13:47'!
nodeChanged
	| node nodeActions |
	node _ browser currentNode ifNil: [^ self].
	nodeActions _ (node metaNode actionsForNode: node) select: [:ea | ea wantsButton].
	actions keys do: 
		[:model | 
		actions at: model put: nil.
		nodeActions do: 
			[:action | model label = action buttonLabel ifTrue: [actions at: model put: action]]].
	actions keys do: [:ea | ea selectionChanged].! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 8/29/2004 11:28'!
push: aModel
	(actions at: aModel) ifNotNilDo: [:action | action trigger]! !

!OBPanel class methodsFor: 'instance creation' stamp: 'cwp 8/31/2004 10:46'!
inBrowser: aBrowser
	^ self new browser: aBrowser! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:58'!
browser
	^ browser! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 8/31/2004 10:45'!
browser: aBrowser
	browser _ aBrowser! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 9/23/2005 23:35'!
dispatcher
	^ browser dispatcher! !

!OBPanel methodsFor: 'testing' stamp: 'cwp 9/16/2004 15:21'!
isDefinitionPanel
	^ false! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 3/14/2004 15:02'!
isDirty
	^ false! !

!OBPanel methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:09'!
isNavigation
	^ false! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 3/9/2004 23:52'!
nodeChanged
	! !

OBPanel subclass: #OBVarButtonPanel
	instanceVariableNames: 'morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBVarButtonPanel commentStamp: 'cwp 12/7/2004 00:21' prior: 0!
OBVarButtonPanel displays a horizontal row of buttons. In contrast to OBFixedButtonPanel, the buttons on display are updated to reflect the actions appropriate to the currently selected node.!

!OBVarButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/27/2004 18:16'!
morph
	^ morph _ (RectangleMorph new)
					layoutPolicy: TableLayout new;
					listDirection: #leftToRight;
					hResizing: #spaceFill;
					vResizing: #spaceFill;
					borderWidth: 0;
					wrapCentering: #center;
					cellPositioning: #leftCenter;
					rubberBandCells: true;
					yourself! !

!OBVarButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/26/2004 22:41'!
morphHeight
	^ 25! !

!OBVarButtonPanel methodsFor: 'updating' stamp: 'cwp 11/27/2004 18:23'!
nodeChanged
	| node actions |
	morph removeAllMorphs.
	node _ browser currentNode ifNil: [^ self].
	actions _ (node metaNode actionsForNode: node) select: [:ea | ea wantsButton].
	actions do: [:ea | morph addMorphBack: ea button]! !

Object subclass: #OBProtocolMock
	instanceVariableNames: 'messages answers exceptions recorder isRecording'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 3/14/2004 15:39'!
answer
	^ answers 
		ifNil: [self]
		ifNotNil: [answers isEmpty 
							ifTrue: [self recorder]
							ifFalse: [answers removeFirst]]! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/28/2004 17:22'!
answer: anObjectOrCollection
	answers ifNil: [answers _ OrderedCollection new].
	anObjectOrCollection isCollection
		ifTrue: [answers addAll: anObjectOrCollection]
		ifFalse: [answers add: anObjectOrCollection]! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:29'!
assert: aBoolean
	self assert: aBoolean message: 'Assertion failed'! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:28'!
assert: aBoolean message: aString
	aBoolean ifFalse: [self fail: aString]! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:30'!
deny: aBoolean
	self assert: aBoolean not! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:29'!
deny: aBoolean message: aString
	self assert: aBoolean not message: aString! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 3/14/2004 15:35'!
do: aBlock
	^ aBlock value: self recorder! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/13/2004 01:41'!
examine: aBlock
	[self do: aBlock]
		on: Exception
		do: [:exception | self handleException: exception].

	self assert: messages isEmpty
		message: 'Expected message not sent'.
	self deny: (exceptions notNil and: [exceptions notEmpty])
		message: 'Expected exception not raised'! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/13/2004 00:51'!
expect: aBlock
	isRecording _ true.
	messages _ OrderedCollection new.
	self do: aBlock.
	isRecording _ false.! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:08'!
fail: aString
	^ TestResult failure sunitSignalWith: aString! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/29/2004 15:49'!
handleException: exception
	| expectation |
	(exception isKindOf: TestFailure) ifTrue: [exception pass].
	(exceptions isNil or: [exceptions isEmpty]) ifTrue: [exception pass].
	expectation _ exceptions removeFirst.
	exception class = expectation first ifFalse: [exception pass].
	exception isResumable ifTrue: [expectation second
									ifNil: [exception resume]
									ifNotNil: [exception resume: expectation second]]! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/28/2004 17:26'!
message: aMessage
	| expected |
	isRecording ifTrue: [messages add: aMessage. ^ self].
	
	self 
		deny: messages isEmpty 
		message: 'No more messages expected'.
	expected _ messages removeFirst.
	self assert: (expected sends: aMessage selector)
		message: 'Unexpected message'.
	self assert: expected arguments = aMessage arguments
		message: 'Incorrect arguments'.
		
	^ self answer! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 3/14/2004 15:36'!
recorder
	^ recorder ifNil: [recorder _ (OBMessageRecorder forMock: self)]! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/28/2004 22:04'!
resume: aNotificationClass 
	^ self resume: aNotificationClass with: nil! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/13/2004 00:41'!
resume: aNotificationClass with: aValue
	exceptions ifNil: [exceptions _ OrderedCollection new].
	exceptions add: (Array with: aNotificationClass with: aValue)! !

!PluggableListMorph methodsFor: '*omnibrowser' stamp: 'cwp 3/14/2004 13:09'!
getListDelicately
	| lazy |
	lazy _ self listMorph.
	^ (1 to: lazy getListSize) collect: [:i | lazy item: i].
	! !

!PluggableListMorph methodsFor: '*omnibrowser' stamp: 'cwp 3/14/2004 13:47'!
getListObtrusively
	| lazy |
	lazy _ self listMorph.
	^ (1 to: lazy getListSize) collect: [:i | lazy getListItem: i].
	! !

TextMorphForEditView subclass: #OBTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBTextMorph commentStamp: 'cwp 12/7/2004 00:07' prior: 0!
This is a trivial subclass of TextMorph. It overrides editor creation to use OBTextMorphEditor rather than a regular TextMorphEditor.!

!OBTextMorph methodsFor: 'private' stamp: 'cwp 10/30/2004 23:06'!
installEditorToReplace: priorEditor
	"Install an editor for my paragraph.  This constitutes 'hasFocus'.
	If priorEditor is not nil, then initialize the new editor from its state.
	We may want to rework this so it actually uses the prior editor."

	| stateArray |
	priorEditor ifNotNil: [stateArray _ priorEditor stateArray].
	editor _ OBTextMorphEditor new morph: self.
	editor changeParagraph: self paragraph.
	priorEditor ifNotNil: [editor stateArrayPut: stateArray].
	self selectionChanged.
	^ editor! !

!LazyListMorph methodsFor: '*omnibrowser' stamp: 'cwp 2/25/2004 21:17'!
rectForRow: index
	"return a rectangle containing the row at index"
	
	| top |
	top _ self top + (index - 1 * font height).
	^ (self left @ top) extent: self width @ font height
! !

TestCase subclass: #OBBrowserTest
	instanceVariableNames: 'model morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/17/2003 15:10'!
isAbstract
	^ self name = 'OBBrowserTest'! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 9/14/2005 22:22'!
assertAListIncludes: anArrayOfStrings
	self listMorphs 
			detect: [:m | | list |
					list _ m getListDelicately collect: [:ea | ea  asString withBlanksTrimmed].
					list includesAllOf: anArrayOfStrings]
			ifNone: [self assert: false].! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:10'!
assertAListMatches: strings
	| list |
	self listMorphs
			detect:	[:m | 
					list _ m getListDelicately.
					(list size = strings size) and: [list includesAllOf: strings]]
			ifNone: [self assert: false].! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/11/2004 23:03'!
assertButtonSelected: aString
	| button |
	button _ (self morphsOfClass: PluggableButtonMorph) 
				detect: [:m | m label contents = aString]
				ifNone: [self signalFailure: 'No button labeled ', aString].
	self assert: button isOn! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 1/8/2005 12:01'!
assertCurrentItemIs: aString
	self assert: self currentItem withBlanksTrimmed = aString! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 12:47'!
assertListAt: index contains: items
	| list |
	list _ self listItemsAt: index.
	self assert: (list size = items size).
	self assert: (list includesAllOf: items).! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 9/14/2005 23:05'!
assertListAt: index includes: items
	| list |
	list _ (self listItemsAt: index) collect: [:ea | ea asString withBlanksTrimmed].
	self assert: (list includesAllOf: items).! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 1/23/2004 22:35'!
assertMenuContains: aString
	| item |
	item _ self menuItemNamed: aString.
	self assert: item notNil! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 12/10/2004 21:39'!
assertNoButtonsVisible
	self assert: ((morph submorphNamed: 'scroller') 
					submorphOfClass: PluggableButtonMorph) isNil.! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/29/2004 19:38'!
assertSelectedTextIs: aString
	| text selection |
	morph _ self textMorph.
	text _ morph getText.
	selection _ morph getSelection.
	self assert: (text atAll: selection) asString = aString.! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/29/2004 19:33'!
assertTextIsSelected
	self assert: self textMorph getSelection size = self textMorph getText size! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/16/2004 20:33'!
assertTextIs: aString
	| actual |
	actual _ self textMorph text.
	self assert: actual = aString.! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/11/2004 23:02'!
clickButtonLabeled: aString
	| button |
	button _ (self morphsOfClass: PluggableButtonMorph) 
				detect: [:m | m label contents = aString]
				ifNone: [self signalFailure: 'No button labeled ', aString].
	button performAction! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 9/14/2005 22:22'!
currentItem
	^ self currentList selection asString! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 5/13/2004 10:59'!
currentList
	^ self listMorphs reversed 
						detect: [:list | list selection notNil]
						ifNone: [self signalFailure: 'No item is currently selected']! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:11'!
denyAListIncludes: anArrayOfStrings
	self listMorphs 
			detect: [:m | m getListDelicately includesAllOf: anArrayOfStrings]
			ifNone: [^ self].
	self assert: false.! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 12/17/2003 15:08'!
deselect: aString
	| listMorph |
	listMorph _ self findListContaining: aString.
	listMorph changeModelSelection: 0.! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drag: nodeName
	| item list |
	item _ StringMorph contents: nodeName asString.
	list _ self findListContaining: nodeName.
	^ list model dragPassengerFor: item inMorph: list! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drag: passangerName to: targetName
	| passanger |
	passanger _ self drag: passangerName.
	self drop: passanger on: targetName! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drop: aPassenger on: targetName
	| list transfer event |
	list _ self findListContaining: targetName.
	self assert: list notNil.
	transfer _ TransferMorph withPassenger: aPassenger.
	transfer dragTransferType: #OmniBrowser.
	event _ self mouseMoveEventIn: targetName.
	self assert: (list model wantsDroppedMorph: transfer event: event inMorph: list).
	^ list model acceptDroppingMorph: transfer event: event inMorph: list! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:52'!
editText: aString
	| textMorph |
	textMorph _ self textMorph.
	textMorph 
		selectAll;
		replaceSelectionWith: aString asText! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 9/14/2005 09:00'!
findListContaining: aString
	^ self listMorphs 
		detect: [:m | | list |
				list _ m getListDelicately collect: [:ea | ea asString withBlanksTrimmed].
				list includes: aString] 
		ifNone: [nil]! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 11/17/2004 23:45'!
forceLayout
	morph bounds: (6@6 corner: 606@406)! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 3/14/2004 13:11'!
listItemsAt: index
	^ (self listMorphs 
		at: index 
		ifAbsent: [self signalFailure: 'List ', index asString, ' does not exist'])
			getListDelicately
	! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
listMorphs
	^ self morphsOfClass: PluggableListMorph! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 1/26/2004 21:53'!
menuItemNamed: aString
	| lists current |
	lists _ self listMorphs.
	current _ lists reversed detect: [:list | list selection notNil].
	^ (current getMenu: true) itemWithWording: aString.
! !

!OBBrowserTest methodsFor: 'running' stamp: 'avi 9/17/2005 01:39'!
metagraph
	^ self browserClass defaultMetaNode! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
morphNamed: aString
	^ morph 
		findDeepSubmorphThat: [:m | m knownName = aString]
		ifAbsent: [self signalFailure: aString, 'not found']! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
morphsOfClass: aClass
	| found |
	found _ OrderedCollection new.
	morph allMorphsDo: [:m | (m isMemberOf: aClass) ifTrue: [found add: m]].
	^ found! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/14/2004 13:14'!
mouseMoveEventIn: name
	| rect plm |
	plm _ self findListContaining: name.
	rect _ plm listMorph rectForRow: (plm getListDelicately indexOf: name).
	^ MouseMoveEvent new
		setType: #mouseEnter
		startPoint: rect center + (rect width @ 0)
		endPoint: rect center
		trail: #()
		buttons: 4
		hand: ActiveHand
		stamp: Time millisecondClockValue! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 1/23/2004 23:30'!
selectMenuItem: aString
	| item |
	item _ self menuItemNamed: aString.
	item ifNil: [self signalFailure: 'Menu item ''', aString, ''' does not exist.'].
	item doButtonAction! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/14/2004 13:46'!
selectSequence: aCollection
	| plm |
	aCollection withIndexDo: [:item :index | 
					plm _ self listMorphs at: index.
					plm changeModelSelection: (plm getListObtrusively indexOf: item)]! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 9/14/2005 09:01'!
select: aString
	"We don't use #getListDelicately because the user would have to scroll through the list
	to find the item we're selecting, thus triggering updates to the list."
	
	| listMorph list |
	listMorph _ self findListContaining: aString.
	self assert: listMorph notNil.
	listMorph model okToChange ifFalse: [^ self].
	list _ listMorph getListObtrusively collect: [:ea | ea asString withBlanksTrimmed].
	listMorph changeModelSelection: (list indexOf: aString)! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 12/17/2003 15:08'!
setText: aString
	| textMorph |
	textMorph _ self textMorph.
	textMorph 
		selectAll;
		replaceSelectionWith: aString asText;
		accept.
	! !

!OBBrowserTest methodsFor: 'running' stamp: 'cwp 12/5/2004 18:16'!
setUp
	model _ self browserClass metaNode: self metagraph node: self node.
	morph _ model morph.
	self forceLayout! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 11/6/2004 23:00'!
textMorph
	^ (self morphsOfClass: OBPluggableTextMorph) last! !

OBBrowserTest subclass: #OBFakeBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFakeBrowserTest methodsFor: 'constants' stamp: 'cwp 12/5/2004 18:18'!
browserClass
	^ OBBrowser! !

!OBFakeBrowserTest methodsFor: 'running' stamp: 'cwp 11/7/2004 21:27'!
metagraph
	^ OBFakeNode metagraph! !

!OBFakeBrowserTest methodsFor: 'running' stamp: 'cwp 12/17/2003 15:12'!
node
	^ OBFakeNode new! !

!OBFakeBrowserTest methodsFor: 'constants' stamp: 'cwp 12/5/2004 18:17'!
paneCount
	^ 4! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 14:25'!
testAcceptText
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'.
	self setText: 'x'.
	
	"This is a gratuitous refresh to make the test pass. It isn't necessary when the 
	browser is actually open. It's a deficiency of the test environment..."
	self listMorphs do: [:ea | ea update: #list].
	
	self assertListAt: 1 contains: #(x b c).
	self assertListAt: 2 contains: #(xa xb xc).
	self assertTextIs: 'x'
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 1/7/2004 08:27'!
testAction
	self select: #a.
	self selectMenuItem:  'become z'.
	self assertListAt: 1 contains:  #(z b c)! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 13:49'!
testConfirmAbandon
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'.
	self editText: 'x'.
	[self select: #b]
		on: OBConfirmationRequest
		do: [:notification | notification cancel].
	self assertCurrentItemIs: #a.
	self assertTextIs: 'x'.
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 11/23/2003 13:12'!
testCreateWindow
	self assert: (morph isKindOf: SystemWindow).
	self assert: (morph label = 'OmniBrowser').
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 8/26/2004 02:17'!
testDeleteB
	self select: #b.
	self selectMenuItem: 'delete'.
	self assertListAt: 1 contains: #(a c).
	self assert: self listMorphs size = self paneCount.
	self assertTextIs: ''.! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 17:20'!
testDeleteRootNode
	self select: #a.
	self selectMenuItem: 'delete root'.
	self denyAListIncludes: #(a b c).
	self assertTextIs: ''! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 8/26/2004 02:17'!
testDeselectA
	self select: #a.
	self deselect: #a.
	self assertListAt: 1 contains: #(a b c).
	self assert: self listMorphs size = self paneCount.
	self assertTextIs: ''.! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 11/16/2004 22:42'!
testDragAndDrop
	| node |
	self select: #a.
	node _ self drag: #ab.
	self assert: (node isKindOf: OBFakeNode).
	self assert: node name = #ab.
	self drop: node on: #c.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(cb).
	self assertCurrentItemIs: #c.
	self assertTextIs: 'c'! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 8/26/2004 02:17'!
testInitialDisplay
	self assert: self listMorphs size = self paneCount.
	self assertListAt: 1 contains: #(a b c).
	self assertTextIs: ''.
	! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:00'!
testPanes
	| scroller |
	self 
		assert: (self morphsOfClass: PluggableListMorph) size = self paneCount.
	self assert: (self morphsOfClass: OBPluggableTextMorph) size = 1.
	scroller := self morphNamed: 'scroller'.
	self assert: scroller panes size = model navigationPanel minPanes! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 9/17/2005 17:44'!
testRefreshDirty
	| root |
	self select: #a.
	self editText: 'x'.
	
	root _ model root.
	root adopt: (OBFakeNode parent: root item: 'z').
	model dispatcher refresh.
	
	self assertListAt: 1 contains: #(a b c z).
	self assertCurrentItemIs: #a.
	self assertTextIs: 'x'.
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/7/2003 19:30'!
testSelectA
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/7/2003 19:30'!
testSelectC
	self select: #c.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #().
	self assertTextIs: 'c'! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'dvf 9/5/2005 18:10'!
testSelectionPath
	self select: #a.
	self select: #ab.
	self assert: ((model selectionPath collect: [:e | e item]) = (OrderedCollection with: 'a' with: 'b'))! !

TestCase subclass: #OBButtonPanelTest
	instanceVariableNames: 'model morph pushed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBButtonPanelTest class methodsFor: 'as yet unclassified' stamp: 'cwp 8/28/2004 20:23'!
isAbstract
	^ self name = #OBButtonPanelTest! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 11:14'!
actionClass
	^ OBAction! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 11:40'!
actions
	^ self buttonLabels collect: [:ea | self actionWithLabel: ea]
! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 11:13'!
actionsForNode: aNode
	^ self actions! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 13:49'!
actionWithLabel: label
	^ OBAction
		label: label asString, 'menu version'
		buttonLabel: label asString
		receiver: self
		selector: #push:
		arguments: (Array with: label)! !

!OBButtonPanelTest methodsFor: 'asserting' stamp: 'cwp 11/27/2004 22:04'!
assertButtonEnabled: index 
	self assert: (self labelIsBlack: index)! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 11:38'!
buttonLabels
	^ #(foo bar baz)! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 3/10/2004 23:08'!
currentNode
	^ (OBFakeNode new) metaNode: self metaNode! !

!OBButtonPanelTest methodsFor: 'callbacks' stamp: 'cwp 11/27/2004 21:52'!
defaultBackgroundColor
	^ Color red! !

!OBButtonPanelTest methodsFor: 'asserting' stamp: 'cwp 11/27/2004 22:04'!
denyButtonEnabled: index 
	self deny: (self labelIsBlack: index)! !

!OBButtonPanelTest methodsFor: 'accessing' stamp: 'cwp 11/27/2004 22:02'!
labelAt: index 
	^(morph submorphs at: index) submorphs first submorphs first! !

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 11/27/2004 22:04'!
labelIsBlack: index 
	^(self labelAt: index) color = Color black! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 3/10/2004 23:09'!
metaNode
	^ OBMetaNode new addActor: self! !

!OBButtonPanelTest methodsFor: 'callbacks' stamp: 'cwp 3/10/2004 23:59'!
push: aSymbol
	pushed _ aSymbol! !

!OBButtonPanelTest methodsFor: 'running' stamp: 'cwp 8/28/2004 20:28'!
setUp
	model _ self panel.
	morph _ model morph.
! !

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 1/8/2005 22:18'!
testLayout
	| rects |
	morph bounds: (10@10 corner: 310@33).
	model nodeChanged.
	
	"This is an ugly hack. I can't figure out how else to force a layout."
	morph openInWorld.
	rects _ morph submorphs collect: [:b | b bounds].
	morph delete.

	rects do: [:ea | self assert: ea top = 10].
	self assert: rects first left = 10.
	self assert: rects first right = rects second left.
	self assert: rects second right = rects third left.
	self assert: rects third right = 310.! !

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 3/10/2004 00:01'!
testPushFoo
	model nodeChanged.
	morph submorphs first performAction.
	self assert: pushed = #foo
	! !

OBButtonPanelTest subclass: #OBFixedButtonPanelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFixedButtonPanelTest methodsFor: 'running' stamp: 'cwp 8/29/2004 11:43'!
actions
	^ (self buttonLabels copyWithout: #bar) collect: [:ea | self actionWithLabel: ea]! !

!OBFixedButtonPanelTest methodsFor: 'running' stamp: 'cwp 8/29/2004 11:42'!
panel
	| panel |
	panel _ OBFixedButtonPanel inBrowser: self.
	self buttonLabels do: [:ea | panel addButtonWithLabel: ea].
	^ panel
	! !

!OBFixedButtonPanelTest methodsFor: 'tests' stamp: 'cwp 11/27/2004 21:58'!
testEnabling
	model nodeChanged.
	self assertButtonEnabled: 1.
	self denyButtonEnabled: 2.
	self assertButtonEnabled: 3.! !

!OBFixedButtonPanelTest methodsFor: 'tests' stamp: 'cwp 8/29/2004 11:44'!
testPushBar
	model nodeChanged.
	morph submorphs second performAction.
	self assert: pushed isNil
	! !

OBButtonPanelTest subclass: #OBVarButtonPanelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBVarButtonPanelTest methodsFor: 'running' stamp: 'cwp 8/28/2004 20:38'!
panel
	^ OBVarButtonPanel inBrowser: self.! !

!OBVarButtonPanelTest methodsFor: 'tests' stamp: 'cwp 8/28/2004 20:25'!
testButtons
	model nodeChanged.
	self assert: morph submorphs size = 3.
	#(foo bar baz) 
		with: morph submorphs 
		do: [:sel :m | self assert: m label contents = sel asString]! !

TestCase subclass: #OBDispatcherTest
	instanceVariableNames: 'dispatcher catcher message foo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBDispatcherTest methodsFor: 'as yet unclassified' stamp: 'cwp 9/18/2005 10:37'!
foo
	foo _ true! !

!OBDispatcherTest methodsFor: 'as yet unclassified' stamp: 'cwp 9/18/2005 10:23'!
message: aMessage
	message _ aMessage! !

!OBDispatcherTest methodsFor: 'as yet unclassified' stamp: 'cwp 9/18/2005 10:37'!
setUp
	dispatcher := OBDispatcher new.
	dispatcher addObserver: self.
	catcher := OBMessageRecorder forMock: self.
	dispatcher addObserver: catcher.
	foo _ false! !

!OBDispatcherTest methodsFor: 'as yet unclassified' stamp: 'cwp 9/18/2005 10:37'!
testForward
	dispatcher foo.
	self assert: foo.
	self assert: message selector = #foo.! !

TestCase subclass: #OBKeyBindingsTest
	instanceVariableNames: 'ptm message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBKeyBindingsTest methodsFor: 'asserting' stamp: 'cwp 10/31/2004 00:36'!
assert: evt sends: aSelector
	ptm keyStroke: evt.
	self assert: message notNil.
	self assert: message selector = aSelector.
	self assert: message arguments size = 1.
	self assert: message arguments first = #OBBrowser.! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/30/2004 23:58'!
browseIt: aSelector
	message _ self thisMessage.
	^ true! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/31/2004 00:39'!
implementorsOfIt: aSelector
	message _ self thisMessage.
	^ true! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:08'!
ksCmdB
	^ self modifier: 64 keycode: 98! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:08'!
ksCmdM
	^ self modifier: 64 keycode: 109! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:08'!
ksCmdN
	^ self modifier: 64 keycode: 110! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:09'!
ksCmdShiftE
	^ self modifier: 72 keycode: 69
! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:09'!
ksCmdShiftN
	^ self modifier: 72 keycode: 78! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:09'!
ksCmdShiftW
	^ self modifier: 72 keycode: 87! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:07'!
modifier: mod keycode: code
	| string |
	string := 'keystroke 988180 ', mod asString, ' ', code asString.
	^ KeyboardEvent readFrom: string readStream! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/31/2004 00:49'!
referencesToIt: aSelector
	message := self thisMessage.
	^ true! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/30/2004 23:26'!
selection
	^ 1 to: 9! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/31/2004 00:25'!
sendersOfIt: aSelector
	message := self thisMessage.
	^ true! !

!OBKeyBindingsTest methodsFor: 'running' stamp: 'cwp 10/30/2004 22:05'!
setUp
	ptm := (OBPluggableTextMorph
			on: self
			text: #text
			accept: #accept:notifying:
			readSelection: #selection
			menu: #menu:shifted:)
				font: Preferences standardCodeFont;
				yourself! !

!OBKeyBindingsTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 00:33'!
testBrowseIt
	self assert: self ksCmdB sends: #browseIt:! !

!OBKeyBindingsTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 00:35'!
testImplementorsOfIt
	self assert: self ksCmdM sends: #implementorsOfIt:! !

!OBKeyBindingsTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 00:44'!
testReferencesToIt
	self assert: self ksCmdShiftN sends: #referencesToIt:! !

!OBKeyBindingsTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 00:35'!
testSendersOfIt
	self assert: self ksCmdN sends: #sendersOfIt:! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/30/2004 23:26'!
text
	^ 'OBBrowser'! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/30/2004 23:57'!
thisMessage
	| context args |
	context := thisContext sender.
	args := Array new: context size.
	1 to: args size do: [:i | args at: i put: (context at: 1)].
	^ Message 
		selector: context method selector
		arguments: args ! !

TestCase subclass: #OBOpenTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

OBOpenTest subclass: #OBLibraryOpenTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBLibraryOpenTest methodsFor: 'accessing' stamp: 'avi 9/17/2005 01:15'!
metagraph
	| fake |
	fake _ OBMetaNode new.
	fake 
		childAt: #children put: fake;
		addActor: (OBNodeActor onNodeClass: OBFakeNode).
	^ fake! !

!OBLibraryOpenTest methodsFor: 'tests' stamp: 'avi 9/17/2005 01:16'!
testGraphRootSelection
	| parent child |
	parent _ OBFakeNode parent: nil item: 'parent'.
	child _ OBFakeNode parent: parent item: 'a'.
	self create: [OBBrowser 
					metaNode: self metagraph
					root: parent
					selection: child].
	self shouldnt: [browser navigationPanel] raise: Error! !

!OBLibraryOpenTest methodsFor: 'tests' stamp: 'avi 9/17/2005 01:17'!
testGraphRootSelectionPanels
	| parent child panels |
	parent _ OBFakeNode parent: nil item: 'parent'.
	child _ OBFakeNode parent: parent item: 'a'.
	panels _ {OBColumnPanel new. OBDefinitionPanel new}.
	self create: [OBBrowser 
					metaNode: self metagraph
					root: parent
					selection: child
					panels: panels].
	self assert: browser navigationPanel minPanes = 1.
	self assert: browser navigationPanel maxPanes = 1.
	self assert: browser root = parent.
	self assert: browser currentNode = child.
	self assert: browser panels size = panels size.
	self assert: (browser panels includesAllOf: panels)! !

!OBOpenTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/5/2004 15:55'!
isAbstract
	^ self name = #OBOpenTest! !

!OBOpenTest methodsFor: 'actions' stamp: 'cwp 12/5/2004 15:54'!
create: aBlock
	self 
		shouldnt: [browser _ aBlock value]
		raise: OBBrowseRequest! !

!OBOpenTest methodsFor: 'actions' stamp: 'cwp 12/5/2004 15:52'!
open: aBlock
	[aBlock value]
		on: OBBrowseRequest
		do: [:n | browser _ n browser].
	self assert: browser notNil! !

TestCase subclass: #OBPaneLayoutTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPaneLayoutTest methodsFor: 'callbacks' stamp: 'cwp 11/2/2004 00:37'!
buttonMorph
	^ RectangleMorph new
		color: Color red;
		yourself! !

!OBPaneLayoutTest methodsFor: 'callbacks' stamp: 'cwp 11/2/2004 00:34'!
listMorph
	^ RectangleMorph new
! !

!OBPaneLayoutTest methodsFor: 'testing' stamp: 'cwp 11/7/2004 00:13'!
testButtonPlacement
	| pane button |
	pane := OBPane forColumn: self withFilter: self.
	pane bounds: (0@0 corner: 200@400).
	button := pane submorphs at: 1.
	self assert: button top = 377.
	self assert: button bottom = 400.
	self assert: button left = pane left.
	self assert: button right = pane right.! !

TestCase subclass: #OBPanelLayoutTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPanelLayoutTest methodsFor: 'asserting' stamp: 'cwp 9/13/2005 21:53'!
assertFrame: frame hasFractions: fraction andOffsets: offset
	self assert: frame leftFraction isNear: fraction left.
	self assert: frame topFraction isNear: fraction top.
	self assert: frame rightFraction isNear: fraction right.
	self assert: frame bottomFraction isNear: fraction bottom.
	
	self assert: frame leftOffset isNear: offset left.
	self assert: frame topOffset isNear: offset top.
	self assert: frame rightOffset isNear: offset right.
	self assert: frame bottomOffset isNear: offset bottom.
! !

!OBPanelLayoutTest methodsFor: 'asserting' stamp: 'cwp 9/13/2005 21:56'!
assert: aNumber isNear: aNumber2 
	self assert: (aNumber - aNumber2) abs <= self tolerance! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 11/7/2004 21:27'!
metaNode
	^ OBFakeNode metagraph! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 8/25/2004 21:44'!
node
	^ OBFakeNode new! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 9/13/2005 21:59'!
testNavPanel
	| window |
	window _ self windowWithPanels: {OBColumnPanel}.
	self assert: window submorphs size >= 2.
	self assert: window submorphs second knownName = 'scroller'.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 12/5/2004 18:28'!
testTextAndButtonPanels
	| window |
	window _ self windowWithPanels: {OBColumnPanel. OBFixedButtonPanel. OBDefinitionPanel}.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@25).
	self
		assertFrame: window submorphs fourth layoutFrame
		hasFractions: (0@0.4 corner: 1@1)
		andOffsets: (0@25 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 9/13/2005 22:00'!
testTextPanel
	| window |
	window _ self windowWithPanels: {OBColumnPanel. OBDefinitionPanel}.
	self assert: window submorphs size >= 3.
	self assert: window submorphs second knownName = 'scroller'.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 12/5/2004 18:29'!
testTwoTextPanels
	"Yeah, ok, this is a bit ridiculous"
	
	| window |
	window _ self windowWithPanels: {OBColumnPanel. OBDefinitionPanel. OBDefinitionPanel}.	
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@0.7)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs fourth layoutFrame
		hasFractions: (0@0.7 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 9/13/2005 21:55'!
tolerance
	^ 4! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 12/5/2004 18:24'!
windowWithPanels: panelClasses
	| browser panels |
	panels := panelClasses collect: [:ea | ea new].
	browser _ OBBrowser 
				metaNode: self metaNode 
				root: self node
				selection: nil
				panels: panels. 
	^ browser morph
! !

TestCase subclass: #OBPaneScrollerTest
	instanceVariableNames: 'paneCount scroller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPaneScrollerTest methodsFor: 'actions' stamp: 'cwp 11/2/2004 00:00'!
createPane
	^ RectangleMorph new
		bounds: (10@10 corner: 20@20);
		yourself! !

!OBPaneScrollerTest methodsFor: 'actions' stamp: 'cwp 11/21/2004 22:40'!
createScroller
	scroller := OBPaneScroller withModel: self.
	scroller bounds: (10 @ 420 extent: 600 @ 160)! !

!OBPaneScrollerTest methodsFor: 'callbacks' stamp: 'cwp 11/21/2004 12:55'!
panes
	^ ((1 to: paneCount) collect: [:i | self createPane]) asOrderedCollection! !

!OBPaneScrollerTest methodsFor: 'callbacks' stamp: 'cwp 11/23/2004 01:37'!
reclaimPanes
	^ scroller panes size - 4! !

!OBPaneScrollerTest methodsFor: 'callbacks' stamp: 'cwp 11/17/2004 23:39'!
sizing
	^ 4! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/21/2004 22:40'!
testPaneHeightNoScrollBar
	paneCount := 4.
	self createScroller.
	self assert: scroller paneHeight = 160! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/21/2004 22:40'!
testPaneHeightWithScrollBar
	paneCount := 5.
	self createScroller.
	self assert: scroller paneHeight = 147! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/23/2004 01:38'!
testPaneReclamation
	paneCount := 6.
	self createScroller.
	scroller submorphs first setValue: 0.2.
	scroller step.
	self assert: scroller panes size = 4! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/21/2004 23:11'!
testResizeUpdatesPaneHeight
	paneCount := 4.
	self createScroller.
	self assert: scroller panes first height = 160.
	scroller bounds: (scroller bounds outsetBy: 1).
	self assert: scroller panes first height = 162! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/21/2004 23:09'!
testResizeUpdatesPaneWidth
	paneCount := 4.
	self createScroller.
	self assert: scroller panes first width = 147.
	scroller bounds: (scroller bounds outsetBy: 2).
	self assert: scroller panes first width = 148! !

TestCase subclass: #OBProtocolMockTest
	instanceVariableNames: 'mock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBProtocolMockTest methodsFor: 'running' stamp: 'cwp 2/13/2004 01:15'!
setUp
	mock _ OBProtocolMock new.! !

!OBProtocolMockTest methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:17'!
shouldFail: aBlock
	self should: aBlock raise: TestResult failure
				
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 17:17'!
testAnswers
	| one two three |
	mock 
		expect: [:stub | stub int; int; int];
		answer: #(1 2 3);
		examine: [:stub | 	one _ stub int.
						two _ stub int.
						three _ stub int].
		self assert: one = 1.
		self assert: two = 2.
		self assert: three = 3! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 15:39'!
testAnswersRecorder
	| one two three |
	mock 
		expect: [:stub | stub int; int; int; foo];
		answer: #(1 2);
		examine: [:stub | one _ stub int.
						two _ stub int.
						three _ stub int.
						three foo].
		self assert: one = 1.
		self assert: two = 2.! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:17'!
testArgFailure
	mock expect: [:stub | stub foo: #bar].
	self shouldFail: [mock examine: [:stub | stub foo: #baz]]
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:17'!
testExtraFailure
	mock expect: [:stub | stub foo].
	self shouldFail: [mock examine: [:stub | stub foo; bar]]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testMissingFailure
	mock expect: [:stub | stub foo; bar].
	self shouldFail: [mock examine: [:stub | stub foo]]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:44'!
testNoExceptionFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self shouldFail: [mock examine: [:stub | stub truth: true]] ! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:20'!
testResume
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true];
		examine: [:stub | stub truth: Notification signal]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testSelectorFailure
	mock expect: [:stub | stub foo].
	self shouldFail: [mock examine: [:stub | stub bar]]
	! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testSuccess
	mock expect: [:stub | stub foo; foo: #bar].
	mock examine: [:stub | stub foo; foo: #bar]
				
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 16:05'!
testTooManyExceptionsFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self should:  [mock examine: [:stub | stub truth: Notification signal. Notification signal]]
		raise: Exception! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:09'!
testWrongExceptionFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self should: [mock examine: [:stub | stub truth: Error signal]] 
		raise: Error! !

TestCase subclass: #OBRadioButtonBarTest
	instanceVariableNames: 'bar selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBRadioButtonBarTest methodsFor: 'asserting' stamp: 'cwp 11/27/2004 18:03'!
assertSelectionIs: anInteger
	self assert: selection = anInteger.
	bar submorphs withIndexDo: 
		[:button :index |
		index = anInteger
			ifTrue: [self assert: button isOn]
			ifFalse: [self deny: button isOn]]! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/22/2004 17:42'!
list
	^ #('alpha' 'beta' 'gamma')! !

!OBRadioButtonBarTest methodsFor: 'simulating' stamp: 'cwp 11/27/2004 18:04'!
push: aString
	| button |
	button _ bar submorphs detect: [:b | b label contents = aString].
	button performAction! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/22/2004 16:39'!
selection
	^ selection! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 19:31'!
selection: anInteger
	selection _ anInteger.
	self changed: #selection! !

!OBRadioButtonBarTest methodsFor: 'running' stamp: 'cwp 11/27/2004 18:03'!
setUp
	selection _ 0.
	bar _ OBRadioButtonBar
			on: self
			list: #list
			selected: #selection
			changeSelected: #selection:.
! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 8/14/2005 14:00'!
testLayout
	| rects |
	
	"This is an ugly hack. I can't figure out how else to force a layout."
	bar openInWorld.
	self changed: #list.
	bar bounds: (100@100 corner: 400@140).
	bar delete.

	rects _ bar submorphs collect: [:b | b bounds].
	self assert: rects first left = bar left.
	self assert: rects first right = rects second left.
	self assert: rects second right = rects third left.
	self assert: rects third right = bar right.
! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/22/2004 16:53'!
testList
	self assert: bar list = self list! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/24/2004 19:35'!
testPushAlphaBeta
	self changed: #list.
	self push: 'alpha'.
	self assertSelectionIs: 1.
	self push: 'beta'.
	self assertSelectionIs: 2.! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 11/27/2004 18:04'!
testUpdateList
	| labels |
	self changed: #list.
	labels _ bar submorphs collect: [:b | b label contents].
	self assert: labels size = self list size.
	self assert: (labels includesAllOf: self list)! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/24/2004 19:15'!
testUpdateSelection
	self changed: #list.
	selection _ 2.
	self changed: #selection.
	self assertSelectionIs: 2.! !

PluggableTextMorph subclass: #OBPluggableTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBPluggableTextMorph commentStamp: 'cwp 12/7/2004 00:04' prior: 0!
This is a trivial subclass of PluggableTextMorph. It overrides initialization methods to use an OBTextMorph rather than a regular TextMorph!

!OBPluggableTextMorph methodsFor: 'event handling' stamp: 'cwp 10/30/2004 23:07'!
keyStroke: evt
	^ textMorph keyStroke: evt! !

!OBPluggableTextMorph methodsFor: 'model access' stamp: 'cwp 10/30/2004 22:57'!
setText: aText
	scrollBar setValue: 0.0.
	textMorph
		ifNil: [textMorph _ OBTextMorph new
						contents: aText wrappedTo: self innerBounds width-6.
				textMorph setEditView: self.
				scroller addMorph: textMorph]
		ifNotNil: [textMorph newContents: aText].
	self hasUnacceptedEdits: false.
	self setScrollDeltas.! !

ProtoObject subclass: #OBMessageRecorder
	instanceVariableNames: 'mock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBMessageRecorder class methodsFor: 'as yet unclassified' stamp: 'cwp 2/13/2004 00:25'!
forMock: aMock
	^ self new obmrSetMock: aMock! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/29/2004 15:52'!
asExplorerString
	^ 'an OBMessageRecorder'! !

!OBMessageRecorder methodsFor: 'recording' stamp: 'cwp 2/28/2004 17:23'!
doesNotUnderstand: aMessage
	^ mock message: aMessage! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/29/2004 15:53'!
hasContentsInExplorer
	^ false! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/13/2004 01:14'!
inspectorClass
	^ Inspector! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 3/17/2004 00:00'!
isMorph
	^ false! !

!OBMessageRecorder methodsFor: 'initializing' stamp: 'cwp 2/13/2004 00:24'!
obmrSetMock: aMockObject
	mock _ aMockObject! !

!OBMessageRecorder methodsFor: 'recording' stamp: 'cwp 9/18/2005 10:25'!
perform: aSelector withArguments: anArray
	mock message: (Message selector: aSelector arguments: anArray)! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 9/18/2005 10:22'!
respondsTo: aSelector
	^ true! !

MessageSend subclass: #OBAction
	instanceVariableNames: 'dispatcher label buttonLabel keystroke icon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBAction commentStamp: 'cwp 3/4/2004 20:52' prior: 0!
Actions represent commands for manipulating the graph domain. They can be made available through menus or buttons in the browser. They carry information on how they should be presented to the user, and are responsible for handling exceptions generated when they are triggered. Actions are created by Actors.

iVars: 

label 		a string which describes the action for the user.
monitor  	when the action is triggered, any notifications raised 
			will be passed to this object for processing!

!OBAction class methodsFor: 'instance creation' stamp: 'cwp 3/10/2004 23:36'!
label: aString buttonLabel: aString2 receiver: anObject selector: aSelector arguments: anArray
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		buttonLabel: aString2! !

!OBAction class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:49'!
label: aString buttonLabel: aString2 receiver: anObject selector: aSelector 
arguments: anArray keystroke: aChar icon: anIcon
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		buttonLabel: aString2;
		keystroke: aChar;
		icon: anIcon! !

!OBAction class methodsFor: 'instance creation' stamp: 'cwp 1/7/2004 09:20'!
label: aString receiver: anObject selector: aSelector arguments: anArray
	^ (self receiver: anObject selector: aSelector arguments: anArray) label: aString! !

!OBAction class methodsFor: 'instance creation' stamp: 'hpt 5/17/2004 16:50'!
label: aString receiver: anObject selector: aSelector arguments: anArray icon: anIcon
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		icon: anIcon! !

!OBAction class methodsFor: 'instance creation' stamp: 'hpt 5/17/2004 14:16'!
label: aString receiver: anObject selector: aSelector arguments: anArray keystroke: aChar
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		keystroke: aChar! !

!OBAction class methodsFor: 'instance creation' stamp: 'hpt 5/17/2004 16:50'!
label: aString receiver: anObject selector: aSelector arguments: anArray keystroke: aChar icon: anIcon
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		keystroke: aChar;
		icon: anIcon! !

!OBAction methodsFor: 'morphic' stamp: 'hpt 5/17/2004 17:00'!
addItemToMenu: aMenu
	aMenu
		add: self labelWithKeystroke
		target: self 
		selector: #trigger.
	Preferences menuWithIcons & self icon notNil
		ifTrue: [aMenu lastItem icon: self icon].
	aMenu addBlankIconsIfNecessary: MenuIcons blankIcon.! !

!OBAction methodsFor: 'morphic' stamp: 'cwp 3/11/2004 22:58'!
button
	^ (PluggableButtonMorph
		on: self
		getState: nil
		action: #trigger
		label: #buttonLabelMorph)	
			onColor: Color lightGray lighter offColor: Color lightGray twiceLighter;
			borderWidth: 2;
			borderRaised;
			hResizing: #spaceFill;
			vResizing: #spaceFill;
			setBalloonText: label! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:35'!
buttonLabel
	^ buttonLabel! !

!OBAction methodsFor: 'morphic' stamp: 'cwp 3/11/2004 22:59'!
buttonLabelMorph
	^ StringMorph 
		contents: self buttonLabel 
		font: Preferences standardButtonFont! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:35'!
buttonLabel: aString
	buttonLabel _ aString
	! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 9/24/2005 00:16'!
dispatcher: aDispatcher
	dispatcher _ aDispatcher! !

!OBAction methodsFor: 'accessing' stamp: 'hpt 5/17/2004 16:42'!
icon
	^icon! !

!OBAction methodsFor: 'accessing' stamp: 'hpt 5/17/2004 16:42'!
icon: anIcon
	icon _ anIcon! !

!OBAction methodsFor: 'accessing' stamp: 'hpt 5/17/2004 14:16'!
keystroke
	^keystroke! !

!OBAction methodsFor: 'accessing' stamp: 'hpt 5/17/2004 14:16'!
keystroke: aChar
	keystroke _ aChar! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 1/7/2004 09:11'!
label
	^ label! !

!OBAction methodsFor: 'morphic' stamp: 'hpt 5/17/2004 14:17'!
labelWithKeystroke
	^keystroke
		ifNil: [label]
		ifNotNil: [label, ' (', keystroke asString, ')']! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 1/7/2004 09:12'!
label: aString
	label _ aString! !

!OBAction methodsFor: 'evaluating' stamp: 'cwp 1/7/2004 18:22'!
perform: aSelector orSendTo: anObject
	^ self perform: aSelector! !

!OBAction methodsFor: 'evaluating' stamp: 'cwp 9/24/2005 00:18'!
trigger
	[self value]
		on: OBDispatcherRequest
		do: [:notification | notification resume: dispatcher].! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:40'!
wantsButton
	^ buttonLabel notNil! !

Slider subclass: #OBScrollBar
	instanceVariableNames: 'upButton downButton pagingArea scrollDelta pageDelta interval menuSelector timeOfMouseDown timeOfLastScroll nextPageDirection currentScrollDelay'
	classVariableNames: 'CachedImages UpArrow UpArrow8Bit'
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBScrollBar commentStamp: 'cwp 3/5/2004 12:01' prior: 0!
This class is a relic. It was originally created to work around bugs in ScrollBar which prevented it from working correctly in horizontal orientation. At some point the bugs should be fixed, and ScrollBar should be used instead.!

!OBScrollBar class methodsFor: 'as yet unclassified' stamp: 'cwp 11/23/2003 18:07'!
alwaysShowFlatScrollbarForAlternativeLook
	"Set this value to true, if you want to see the flat scrollbar look in flop-out mode as well as inboard.  Otherwise the flop-out scrollbar will be rounded and inboard will be flat."
	^ false! !

!OBScrollBar class methodsFor: 'class initialization' stamp: 'cwp 11/23/2003 18:07'!
initialize "ScrollBar initialize"

	UpArrow _ Form
		extent: 6@3
		fromArray: #(2r11e28 2r1111e27 2r111111e26)
		offset: 0@0.! !

!OBScrollBar methodsFor: 'accessing' stamp: 'cwp 11/23/2003 18:06'!
adoptPaneColor: aColor
	"Adopt the given pane color"
	self alternativeScrollbarLook ifFalse:[^self].
	aColor ifNil:[^self].
	self sliderColor: aColor.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
alternativeScrollbarLook
	^Preferences alternativeScrollbarLook! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
buttonExtent
	^ bounds isWide
		ifTrue: [11 @ self innerBounds height]
		ifFalse: [self innerBounds width @ 11]! !

!OBScrollBar methodsFor: 'accessing' stamp: 'cwp 11/23/2003 18:06'!
cachedImageAt: aKey ifAbsentPut: aBlock

	CachedImages ifNil: [CachedImages _ Dictionary new].
	^CachedImages at: aKey ifAbsentPut: aBlock! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollByPage
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 300 delay2: 100) ifFalse: [^ self].
	nextPageDirection
		ifTrue: [self setValue: (value + pageDelta min: 1.0)]
		ifFalse: [self setValue: (value - pageDelta max: 0.0)]
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollDown
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value + scrollDelta + 0.000001 min: 1.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollUp
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value - scrollDelta - 0.000001 max: 0.0)! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
expandSlider
	"Compute the new size of the slider (use the old sliderThickness as a minimum)."
	| r |
	r _ self totalSliderArea.
	slider extent: (bounds isWide
		ifTrue: [((r width * interval) asInteger max: self sliderThickness) @ slider height]
		ifFalse: [slider width @ ((r height * interval) asInteger max: self sliderThickness)])! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
extent: p
        p x > p y
                ifTrue: [super extent: (p max: 42@8)]
                ifFalse: [super extent: (p max: 8@42)]! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
finishedScrolling
	self stopStepping.
	self scrollBarAction: nil.
	self roundedScrollbarLook ifTrue:[
		upButton borderStyle: (BorderStyle complexRaised width: upButton borderWidth).
		downButton borderStyle: (BorderStyle complexRaised width: downButton borderWidth).
	] ifFalse:[
		downButton borderRaised.
		upButton borderRaised.
	].

! !

!OBScrollBar methodsFor: 'initialization' stamp: 'cwp 11/23/2003 18:07'!
initialize
	super initialize.
	scrollDelta _ 0.02.
	pageDelta _ 0.2.
	self roundedScrollbarLook ifTrue:[
		self borderStyle: ((BorderStyle complexFramed width: 2) "baseColor: Color gray")].! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeDownButton
	downButton := RectangleMorph
		newBounds: (self innerBounds bottomRight - self buttonExtent extent: self buttonExtent)
		color: self thumbColor.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	downButton addMorphCentered: (ImageMorph new image: 
		(self 
			cachedImageAt: (bounds isWide ifTrue: ['right'] ifFalse: ['down']) 
			ifAbsentPut: [
				self upArrow8Bit
					rotateBy: (bounds isWide ifTrue: [#right] ifFalse: [#pi])
					centerAt: 0@0
			]
		)
	).
	self roundedScrollbarLook ifTrue:[
		downButton color: Color veryLightGray.
		downButton borderStyle: (BorderStyle complexRaised width: 3).
	] ifFalse:[
		downButton setBorderWidth: 1 borderColor: #raised.
	].
	self addMorph: downButton.
! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeEmbedded: aBool
	"aBool == true => inboard scrollbar
	aBool == false => flop-out scrollbar"
	self roundedScrollbarLook ifFalse:[^self].
	aBool ifTrue:[
		self borderStyle: (BorderStyle inset width: 2).
		self cornerStyle: #square.
	] ifFalse:[
		self borderStyle: (BorderStyle width: 1 color: Color black).
		self cornerStyle: #rounded.
	].
	self removeAllMorphs.
	self initializeSlider.! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializePagingArea
	pagingArea := RectangleMorph newBounds: self totalSliderArea
								color: (Color r: 0.6 g: 0.6 b: 0.8).
	pagingArea borderWidth: 0.
	pagingArea on: #mouseDown send: #scrollPageInit: to: self.
	pagingArea on: #mouseUp send: #finishedScrolling to: self.
	self addMorph: pagingArea.
	self roundedScrollbarLook 
		ifTrue:[pagingArea color: (Color gray: 0.9)].! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeSlider
	self roundedScrollbarLook ifTrue:[
		self 
			initializeUpButton;
			initializeDownButton;
			initializePagingArea.
	] ifFalse:[
		self initializeUpButton; 
			initializeDownButton; 
			initializePagingArea.
	].
	super initializeSlider.
	self roundedScrollbarLook ifTrue:[
		slider cornerStyle: #rounded.
		slider borderStyle: (BorderStyle complexRaised width: 3).
		sliderShadow cornerStyle: #rounded.
	].
	self sliderColor: self sliderColor.! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeUpButton
	upButton := self roundedScrollbarLook 
				ifTrue:  
					[RectangleMorph 
						newBounds: (self innerBounds topLeft extent: self buttonExtent)]
				ifFalse: 
					[RectangleMorph 
						newBounds: ((self innerBounds topLeft) extent: self buttonExtent)].
	upButton color: self thumbColor.
	upButton 
		on: #mouseDown
		send: #scrollUpInit
		to: self.
	upButton 
		on: #mouseUp
		send: #finishedScrolling
		to: self.
	upButton 
		addMorphCentered: (ImageMorph new image: (self 
						cachedImageAt: (bounds isWide ifTrue: ['left'] ifFalse: ['up'])
						ifAbsentPut: 
							[bounds isWide 
								ifTrue: [self upArrow8Bit rotateBy: #left centerAt: 0 @ 0]
								ifFalse: [self upArrow8Bit]])).
	self roundedScrollbarLook 
		ifTrue: 
			[upButton color: Color veryLightGray.
			upButton borderStyle: (BorderStyle complexRaised width: 3)]
		ifFalse: [upButton setBorderWidth: 1 borderColor: #raised].
	self addMorph: upButton! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
interval: d
	"Supply an optional floating fraction so slider can expand to indicate range"
	interval _ d min: 1.0.
	self expandSlider.
	self computeSlider.! !

!OBScrollBar methodsFor: 'other events' stamp: 'cwp 11/23/2003 18:06'!
menuButtonMouseDown: event
	event hand showTemporaryCursor: nil.
	self use: menuSelector orMakeModelSelectorFor: 'MenuButtonPressed:'
		in: [:sel | menuSelector _ sel.  model perform: sel with: event]! !

!OBScrollBar methodsFor: 'other events' stamp: 'cwp 11/23/2003 18:06'!
mouseDownInSlider: event
	interval = 1.0 ifTrue:
		["make the entire scrollable area visible if a full scrollbar is clicked on"
		self setValue: 0.
		self model hideOrShowScrollBar].
	super mouseDownInSlider: event! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
pagingArea
	^pagingArea! !

!OBScrollBar methodsFor: 'scroll timing' stamp: 'cwp 11/23/2003 18:07'!
resetTimer
	timeOfMouseDown _ Time millisecondClockValue.
	timeOfLastScroll _ timeOfMouseDown - 1000 max: 0.
	nextPageDirection _ nil.
	currentScrollDelay _ nil! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
roundedScrollbarLook
	"Rounded look currently only shows up in flop-out mode"
	^Preferences alternativeScrollbarLook and: [Preferences inboardScrollbars not and: [self class alwaysShowFlatScrollbarForAlternativeLook not]]
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollBarAction
	^self valueOfProperty: #scrollBarAction! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollBarAction: aSymbol
	self setProperty: #scrollBarAction toValue: aSymbol! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
scrollDelta
	^ scrollDelta! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
scrollDelta: d1 pageDelta: d2
	"Supply optional increments for better scrolling of, eg, text"
	scrollDelta _ d1.
	pageDelta _ d2.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDown
	self flag: #obsolete.
	downButton eventHandler: nil.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollDownInit! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDownInit
	downButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollDown.
	self startStepping.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDown: count
	self setValue: (value + (scrollDelta * count) + 0.000001 min: 1.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollPageInit: evt
	self resetTimer.
	self setNextDirectionFromEvent: evt.
	self scrollBarAction: #doScrollByPage.
	self startStepping.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUp
	self flag: #obsolete.
	upButton eventHandler: nil.
	upButton on: #mouseDown send: #scrollUpInit to: self.
	upButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollUpInit! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUpInit
	upButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollUp.
	self startStepping.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUp: count
	self setValue: (value - (scrollDelta * count) - 0.000001 max: 0.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
setNextDirectionFromEvent: event

	nextPageDirection _ bounds isWide ifTrue: [
		event cursorPoint x >= slider center x
	]
	ifFalse: [
		event cursorPoint y >= slider center y
	]

! !

!OBScrollBar methodsFor: 'model access' stamp: 'cwp 11/23/2003 18:06'!
setValue: newValue
	"Using roundTo: instead of truncateTo: ensures that scrollUp will scroll the same distance as scrollDown."
	^ super setValue: (newValue roundTo: scrollDelta)! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
sliderColor: aColor
	"Change the color of the scrollbar to go with aColor."
	| buttonColor |
	super sliderColor: aColor.
	buttonColor _ self thumbColor.
	upButton color: buttonColor.
	downButton color: buttonColor.
	slider color: buttonColor.
	self alternativeScrollbarLook ifTrue:
		[self roundedScrollbarLook
			ifTrue:
				[self color: Color transparent.
				pagingArea color: aColor muchLighter.
				self borderStyle style == #simple 
					ifTrue:[self borderColor: aColor darker darker]
					ifFalse:[self borderStyle baseColor: aColor]]
			ifFalse:
				[pagingArea color: (aColor alphaMixed: 0.3 with: Color white).
				self borderWidth: 0]]
! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
sliderExtent
	"The sliderExtent is now stored in the slider itself, not hardcoded as it is in the superclass."
	^slider extent! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
sliderShadowColor
	^ self roundedScrollbarLook
		ifTrue: [self sliderColor darker]
		ifFalse: [super sliderShadowColor]
! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:07'!
sliderThickness
	^ self roundedScrollbarLook ifTrue:[15] ifFalse:[super sliderThickness]! !

!OBScrollBar methodsFor: 'stepping and presenter' stamp: 'cwp 11/23/2003 18:07'!
step
	| action |
	action _ self scrollBarAction.
	action ifNotNil:[self perform: action].! !

!OBScrollBar methodsFor: 'testing' stamp: 'cwp 11/23/2003 18:06'!
stepTime
	^ currentScrollDelay ifNil: [300]! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
thumbColor
	"Problem: Part of the ScrollBar/Slider code uses 'slider' to mean the entire scrollbar/slider widget, and part of it uses 'slider' to mean only the draggable 'thumb'.  This should be cleaned up so that 'thumb' is used instead of 'slider' where appropriate.  For now, the meaning of thumbColor is clear, at least."

	^ self alternativeScrollbarLook 
		ifTrue: [self sliderColor alphaMixed: 0.7 with: (Color gray: 0.95)]
		ifFalse: [Color veryLightGray]! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:07'!
totalSliderArea
	| upperBoundsButton |
	upperBoundsButton _ upButton.
	upButton bottom > upperBoundsButton bottom
		ifTrue: [upperBoundsButton _ upButton].
	^ bounds isWide
		ifTrue: [upperBoundsButton bounds topRight corner: downButton bounds bottomLeft]
		ifFalse: [upperBoundsButton bounds bottomLeft corner: downButton bounds topRight].
! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
upArrow8Bit

	"convert to 8-bit and convert white to transparent to avoid gratuitous conversion every time we put one in an ImageMorph"

	^UpArrow8Bit ifNil: [
		UpArrow8Bit _ (ColorForm mappingWhiteToTransparentFrom: UpArrow) asFormOfDepth: 8
	]! !

!OBScrollBar methodsFor: 'scroll timing' stamp: 'cwp 11/23/2003 18:07'!
waitForDelay1: delay1 delay2: delay2 
	"Return true if an appropriate delay has passed since the last scroll operation.
	The delay decreases exponentially from delay1 to delay2."

	| now scrollDelay |
	timeOfLastScroll isNil ifTrue: [self resetTimer].	"Only needed for old instances"
	now := Time millisecondClockValue.
	(scrollDelay := currentScrollDelay) isNil 
		ifTrue: [scrollDelay := delay1	"initial delay"].
	currentScrollDelay := scrollDelay * 9 // 10 max: delay2.	"decrease the delay"
	timeOfLastScroll := now.
	^true! !

!OBScrollBar methodsFor: 'testing' stamp: 'cwp 11/23/2003 18:06'!
wantsSteps
	^self scrollBarAction notNil! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/4/2004 00:10'!
longNameOf: shortName
	^ shortName isEmpty
		ifTrue: [self systemCategoryPrefix]
		ifFalse: [self systemCategoryPrefix, '-', shortName]
! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/4/2004 00:08'!
shortNameOf: aSystemCategoryName
	^ (aSystemCategoryName = self systemCategoryPrefix)
		ifTrue: ['']
		ifFalse: [aSystemCategoryName allButFirst: self systemCategoryPrefix size + 1]
! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/17/2004 22:19'!
starLongNameOf: shortName
	^ shortName isEmpty
		ifTrue: [self methodCategoryPrefix]
		ifFalse: [self methodCategoryPrefix, '-', shortName]
! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/17/2004 22:14'!
starShortNameOf: aMethodCategoryName
	^ (aMethodCategoryName = self methodCategoryPrefix)
		ifTrue: ['']
		ifFalse: [aMethodCategoryName allButFirst: self methodCategoryPrefix size + 1]
! !

!PluggableButtonMorph methodsFor: '*omnibrowser' stamp: 'cwp 2/24/2004 19:07'!
isOn
	^ self color = onColor! !

Notification subclass: #OBDispatcherRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

Notification subclass: #OBInteractionRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBInteractionRequest commentStamp: 'cwp 12/7/2004 00:13' prior: 0!
OBInteractionRequest is an abstract superclass for notifications that request some interaction with the user. It's useful for catching such notifications in an exception handler, while allowing other notifications to operate normally.!

OBInteractionRequest subclass: #OBBrowseRequest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBBrowseRequest commentStamp: 'cwp 12/7/2004 00:10' prior: 0!
This notification is raised whenever a browser needs to be opened. The default action is to open a SystemWindow in Morphic, but it can be caught in situations where that's not appropriate. The OB test suite uses this extensively. !

!OBBrowseRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2004 18:45'!
signal: aBrowser
	^ (self new browser: aBrowser) signal! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 18:45'!
browser
	^ browser! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 18:45'!
browser: aBrowser
	browser _ aBrowser! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 21:27'!
defaultAction
	^ browser openInMorphic! !

!OBBrowseRequest methodsFor: 'testing' stamp: 'cwp 10/17/2004 19:32'!
isBrowseRequest
	^ true! !

OBInteractionRequest subclass: #OBChoiceRequest
	instanceVariableNames: 'prompt labels values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBChoiceRequest commentStamp: 'cwp 3/5/2004 12:26' prior: 0!
This notification is used to ask the user for to choose from a list of alternatives. Its defaultAction is to open a PopUpMenu. Test cases an intercept the notification and respond programmatically.

prompt	- A string describing the choice the user is asked to make.
labels	- A list of strings describing the alternatives.
values	- When the user chooses an alternative, the corresponding item from this list is returned!

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 3/4/2004 02:05'!
labels: anArray
	^ (self new setPrompt: nil labels: anArray values: anArray) signal! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 3/4/2004 02:27'!
prompt: aString labels: labelArray values: valueArray
	^ (self new setPrompt: aString labels: labelArray values: valueArray) signal! !

!OBChoiceRequest methodsFor: 'signaling' stamp: 'cwp 5/11/2004 11:51'!
defaultAction
	^ values 
		at: ((PopUpMenu labelArray: labels) startUpWithCaption: prompt)
		ifAbsent: [nil]! !

!OBChoiceRequest methodsFor: 'initializing' stamp: 'cwp 3/4/2004 02:03'!
setPrompt: aString labels: labelArray values: valueArray
	prompt _ aString.
	labels _ labelArray.
	values _ valueArray! !

OBInteractionRequest subclass: #OBConfirmationRequest
	instanceVariableNames: 'prompt confirm cancel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBConfirmationRequest commentStamp: 'cwp 3/5/2004 12:30' prior: 0!
This notification is used to ask the user to confirm some kind potentially dangerous operation. Its default action is to open a PopUpMenu.

iVars:

prompt		- a string describing the situation the user is asked to confirm
confirm		- a string describing the action that will be taken if the user confirms
cancel		- a string describing the action that will be taken if the user does not confirm!

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:09'!
newPrompt: prompt confirm: confirm cancel: cancel
	^ self new setPrompt: prompt confirm: confirm cancel: cancel! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:13'!
prompt: prompt
	^ self prompt: prompt confirm: 'Ok'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:12'!
prompt: prompt confirm: confirm
	^ self prompt: prompt confirm: confirm cancel: 'Cancel'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:02'!
prompt: prompt confirm: confirm cancel: cancel
	^ (self newPrompt: prompt confirm: confirm cancel: cancel) signal! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
cancel
	self resume: false! !

!OBConfirmationRequest methodsFor: 'user interaction' stamp: 'cwp 2/28/2004 12:08'!
defaultAction
	^ PopUpMenu confirm: prompt trueChoice: confirm falseChoice: cancel! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
ok
	self resume: true! !

!OBConfirmationRequest methodsFor: 'initalizing' stamp: 'cwp 2/28/2004 12:00'!
setPrompt: promptString confirm: confirmString cancel: cancelString
	prompt _ promptString.
	confirm _ confirmString.
	cancel _ cancelString! !

!OBInteractionRequest methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2004 19:31'!
isBrowseRequest
	^ false! !

OBInteractionRequest subclass: #OBTextRequest
	instanceVariableNames: 'prompt template'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBTextRequest commentStamp: 'cwp 3/5/2004 12:35' prior: 0!
This notification is used to ask the user to supply a short piece of text. Its defaultAction is to open a FillInTheBlank.

iVars:

prompt		- a string describing the text the user is asked to supply
template	- a default reply !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/2/2004 21:30'!
prompt: aString
	^ self new prompt: aString; template: ''! !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/5/2004 20:31'!
prompt: promptString template: templateString
	^ (self new prompt: promptString; template: templateString) signal! !

!OBTextRequest methodsFor: 'user interaction' stamp: 'cwp 2/2/2004 21:27'!
defaultAction
	self resume: self requestText! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:24'!
prompt
	^ prompt! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:23'!
prompt: aString
	prompt _ aString! !

!OBTextRequest methodsFor: 'user interaction' stamp: 'cwp 2/2/2004 21:27'!
requestText
	^ FillInTheBlankMorph 
		request: prompt
		initialAnswer: template
		centerAt: Sensor cursorPoint
		inWorld: World
		onCancelReturn: nil
		acceptOnCR: true! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:00'!
template
	^ template! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 20:59'!
template: aString
	template _ aString! !
OBScrollBar initialize!
